from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="TransitIncident")


@_attrs_define
class TransitIncident:
    """An incident describes disruptions on the transit network.
    Disruptions scale from delays to service cancellations.

        Example:
            {'summary': 'The subway is closed each night between 1 AM and 5 AM.', 'description': 'The subway is closed each
                night between 1 AM and 5 AM while we clean our trains and stations. We are running extra bus service
                overnight.', 'type': 'maintenance', 'effect': 'modifiedService'}

        Attributes:
            type_ (str): Extensible enum: `technicalProblem` `strike` `demonstration` `accident` `holiday` `weather`
                `maintenance` `construction` `policeActivity` `medicalEmergency` `other` `...`
                An open list of possible incident causes / types.
                Note: Since new types are expected to appear, it is important to check for unknown types when parsing this
                value.
            effect (str): Extensible enum: `cancelledService` `reducedService` `additionalService` `modifiedService`
                `delays` `detour` `stopMoved` `other` `...`
                An open list of possible incident effects.
                Note: Since new types are expected to appear, it is important to check for unknown types when parsing this
                value.
            summary (str | Unset): A human readable summary of the incident Example: The subway is closed each night between
                1 AM and 5 AM..
            description (str | Unset): A human readable description of the incident Example: The subway is closed each night
                between 1 AM and 5 AM while we clean our trains and stations. We are running extra bus service overnight..
            valid_from (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either `date-time` or `date-
                only` 'T' `partial-time` (ie no time-offset).
            valid_until (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either `date-time` or `date-
                only` 'T' `partial-time` (ie no time-offset).
            url (str | Unset): An URL address that links to a particular resource. Example:
                https://url.address.com/resource.
    """

    type_: str
    effect: str
    summary: str | Unset = UNSET
    description: str | Unset = UNSET
    valid_from: datetime.datetime | Unset = UNSET
    valid_until: datetime.datetime | Unset = UNSET
    url: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        effect = self.effect

        summary = self.summary

        description = self.description

        valid_from: str | Unset = UNSET
        if not isinstance(self.valid_from, Unset):
            valid_from = self.valid_from.isoformat()

        valid_until: str | Unset = UNSET
        if not isinstance(self.valid_until, Unset):
            valid_until = self.valid_until.isoformat()

        url = self.url

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "effect": effect,
        })
        if summary is not UNSET:
            field_dict["summary"] = summary
        if description is not UNSET:
            field_dict["description"] = description
        if valid_from is not UNSET:
            field_dict["validFrom"] = valid_from
        if valid_until is not UNSET:
            field_dict["validUntil"] = valid_until
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        effect = d.pop("effect")

        summary = d.pop("summary", UNSET)

        description = d.pop("description", UNSET)

        _valid_from = d.pop("validFrom", UNSET)
        valid_from: datetime.datetime | Unset
        if isinstance(_valid_from, Unset):
            valid_from = UNSET
        else:
            valid_from = isoparse(_valid_from)

        _valid_until = d.pop("validUntil", UNSET)
        valid_until: datetime.datetime | Unset
        if isinstance(_valid_until, Unset):
            valid_until = UNSET
        else:
            valid_until = isoparse(_valid_until)

        url = d.pop("url", UNSET)

        transit_incident = cls(
            type_=type_,
            effect=effect,
            summary=summary,
            description=description,
            valid_from=valid_from,
            valid_until=valid_until,
            url=url,
        )

        transit_incident.additional_properties = d
        return transit_incident

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.match_info_qq import MatchInfoQq
from ..types import UNSET, Unset

T = TypeVar("T", bound="MatchInfo")


@_attrs_define
class MatchInfo:
    """
    Attributes:
        start (int): First index of the matched range (0-based indexing, inclusive)
        end (int): One past the last index of the matched range (0-based indexing, exclusive); The difference between
            end and start gives the length of the term
        value (str): Matched term in the input string
        qq (MatchInfoQq | Unset): The matched qualified query field type. If this is not returned, then matched value
            refers to the freetext query
    """

    start: int
    end: int
    value: str
    qq: MatchInfoQq | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        start = self.start

        end = self.end

        value = self.value

        qq: str | Unset = UNSET
        if not isinstance(self.qq, Unset):
            qq = self.qq.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "start": start,
            "end": end,
            "value": value,
        })
        if qq is not UNSET:
            field_dict["qq"] = qq

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        start = d.pop("start")

        end = d.pop("end")

        value = d.pop("value")

        _qq = d.pop("qq", UNSET)
        qq: MatchInfoQq | Unset
        if isinstance(_qq, Unset):
            qq = UNSET
        else:
            qq = MatchInfoQq(_qq)

        match_info = cls(
            start=start,
            end=end,
            value=value,
            qq=qq,
        )

        match_info.additional_properties = d
        return match_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import argparse

import pandas as pd

# noinspection PyProtectedMember
from casageo.tools import CasaGeoClient, CasaGeoError, _consts, _util

# noinspection PyProtectedMember
from casageo.tools._coder import AddressResult, CasaGeoCoder

__all__ = [
    "AddressResult",
    "CasaGeoCoder",
]


def _main(args: argparse.Namespace) -> dict | pd.DataFrame:
    cga = CasaGeoClient(args.key, preferred_language=args.lang)
    cgc = CasaGeoCoder(cga)

    if args.get_info:
        return cga.account_info().json()
    elif args.addrstr:
        result = cgc.address(args.addrstr)
        if args.json:
            return result.json()
        else:
            return result.dataframe()
    elif args.file:
        df = pd.read_csv(args.file, sep=";")
        if args.format:
            column_format = args.format.split(",")
            if len(column_format) != len(df.columns):
                raise CasaGeoError(
                    "The amount of columns in format and data must be equal",
                )

            if not set(column_format).issubset(_consts.ALLOWED_COLUMNS):
                raise CasaGeoError(
                    f"The format must only contain {', '.join(_consts.ALLOWED_COLUMNS)}.",
                )

            dupes = _util.duplicates(column_format)
            while "_" in dupes:
                dupes.remove("_")
            if dupes:
                raise CasaGeoError(
                    "There cannot be multiple columns of the same name except for _",
                )

            # if there is more than one _ rename them to _1, _2 etc
            for i, name in enumerate(column_format):
                if name == "_":
                    column_format[i] = f"_{i}"

            df.columns = column_format

        if "id" not in df.columns:
            raise CasaGeoError("DataFrame must contain an 'id' column")

        if df["id"].duplicated().any():
            raise CasaGeoError("'id' column contains duplicate values")

        if args.json:
            return cgc._bulk_address_json(df)
        else:
            raise CasaGeoError("Dataframe output is currently not supported")
    else:
        raise CasaGeoError("Please provide an argument")


if __name__ == "__main__":
    _util.cli_main(_consts.get_coder_parser(), _main)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.simple_turn_action_action import SimpleTurnActionAction
from ..models.turn_action_direction import TurnActionDirection
from ..models.turn_action_severity import TurnActionSeverity
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.exit_info import ExitInfo
    from ..models.localized_string import LocalizedString
    from ..models.road_info import RoadInfo
    from ..models.signpost_info import SignpostInfo


T = TypeVar("T", bound="SimpleTurnAction")


@_attrs_define
class SimpleTurnAction:
    """Turn maneuver, such as "Make a sharp right turn at"

    Attributes:
        action (SimpleTurnActionAction): The type of the action.

            **NOTE:** The list of possible actions may be extended in the future. The client application should handle such
            a case gracefully.
        duration (int): Duration in seconds. Example: 198.
        instruction (str | Unset): Description of the action (e.g. Turn left onto Minna St.).
        offset (int | Unset): Offset of a coordinate in the section's polyline.
        length (int | Unset): Distance in meters. Example: 189.
        current_road (RoadInfo | Unset): Road information attached to an offset action Example: {'fennstrasse': {'type':
            'street', 'name': [{'value': 'Fennstraße', 'language': 'de'}], 'number': [{'value': 'B96', 'language': 'de'}],
            'toward': [{'value': 'Reinickendorf', 'language': 'de'}]}}.
        next_road (RoadInfo | Unset): Road information attached to an offset action Example: {'fennstrasse': {'type':
            'street', 'name': [{'value': 'Fennstraße', 'language': 'de'}], 'number': [{'value': 'B96', 'language': 'de'}],
            'toward': [{'value': 'Reinickendorf', 'language': 'de'}]}}.
        exit_sign (ExitInfo | Unset): Exit information attached to an offset action Example: {'exit': {'number':
            [{'value': '15', 'language': 'de'}]}}.
        signpost (SignpostInfo | Unset): Signpost information attached to an offset action.
             Example: {'$ref': '#/components/examples/routeResponseManeuverSignpostInfoExample'}.
        intersection_name (list[LocalizedString] | Unset): Name of the intersection where the turn takes place, if
            available.
        direction (TurnActionDirection | Unset): Direction of the turn. `middle` is only used by the `keep` action.
        severity (TurnActionSeverity | Unset): Tightness of the turn. Optional in the `turn` action, unused by other
            actions.

            * `light`: indicates making a light turn
            * `quite`: indicates making a regular turn
            * `heavy`: indicates making a heavy turn
        turn_angle (float | Unset): The angle of the turn expressed in degrees.

            It is available only for turn-by-turn actions. A positive number indicates a clockwise angle.
             Example: -132.28.
    """

    action: SimpleTurnActionAction
    duration: int
    instruction: str | Unset = UNSET
    offset: int | Unset = UNSET
    length: int | Unset = UNSET
    current_road: RoadInfo | Unset = UNSET
    next_road: RoadInfo | Unset = UNSET
    exit_sign: ExitInfo | Unset = UNSET
    signpost: SignpostInfo | Unset = UNSET
    intersection_name: list[LocalizedString] | Unset = UNSET
    direction: TurnActionDirection | Unset = UNSET
    severity: TurnActionSeverity | Unset = UNSET
    turn_angle: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        action = self.action.value

        duration = self.duration

        instruction = self.instruction

        offset = self.offset

        length = self.length

        current_road: dict[str, Any] | Unset = UNSET
        if not isinstance(self.current_road, Unset):
            current_road = self.current_road.to_dict()

        next_road: dict[str, Any] | Unset = UNSET
        if not isinstance(self.next_road, Unset):
            next_road = self.next_road.to_dict()

        exit_sign: dict[str, Any] | Unset = UNSET
        if not isinstance(self.exit_sign, Unset):
            exit_sign = self.exit_sign.to_dict()

        signpost: dict[str, Any] | Unset = UNSET
        if not isinstance(self.signpost, Unset):
            signpost = self.signpost.to_dict()

        intersection_name: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.intersection_name, Unset):
            intersection_name = []
            for intersection_name_item_data in self.intersection_name:
                intersection_name_item = intersection_name_item_data.to_dict()
                intersection_name.append(intersection_name_item)

        direction: str | Unset = UNSET
        if not isinstance(self.direction, Unset):
            direction = self.direction.value

        severity: str | Unset = UNSET
        if not isinstance(self.severity, Unset):
            severity = self.severity.value

        turn_angle = self.turn_angle

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "action": action,
            "duration": duration,
        })
        if instruction is not UNSET:
            field_dict["instruction"] = instruction
        if offset is not UNSET:
            field_dict["offset"] = offset
        if length is not UNSET:
            field_dict["length"] = length
        if current_road is not UNSET:
            field_dict["currentRoad"] = current_road
        if next_road is not UNSET:
            field_dict["nextRoad"] = next_road
        if exit_sign is not UNSET:
            field_dict["exitSign"] = exit_sign
        if signpost is not UNSET:
            field_dict["signpost"] = signpost
        if intersection_name is not UNSET:
            field_dict["intersectionName"] = intersection_name
        if direction is not UNSET:
            field_dict["direction"] = direction
        if severity is not UNSET:
            field_dict["severity"] = severity
        if turn_angle is not UNSET:
            field_dict["turnAngle"] = turn_angle

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.exit_info import ExitInfo
        from ..models.localized_string import LocalizedString
        from ..models.road_info import RoadInfo
        from ..models.signpost_info import SignpostInfo

        d = dict(src_dict)
        action = SimpleTurnActionAction(d.pop("action"))

        duration = d.pop("duration")

        instruction = d.pop("instruction", UNSET)

        offset = d.pop("offset", UNSET)

        length = d.pop("length", UNSET)

        _current_road = d.pop("currentRoad", UNSET)
        current_road: RoadInfo | Unset
        if isinstance(_current_road, Unset):
            current_road = UNSET
        else:
            current_road = RoadInfo.from_dict(_current_road)

        _next_road = d.pop("nextRoad", UNSET)
        next_road: RoadInfo | Unset
        if isinstance(_next_road, Unset):
            next_road = UNSET
        else:
            next_road = RoadInfo.from_dict(_next_road)

        _exit_sign = d.pop("exitSign", UNSET)
        exit_sign: ExitInfo | Unset
        if isinstance(_exit_sign, Unset):
            exit_sign = UNSET
        else:
            exit_sign = ExitInfo.from_dict(_exit_sign)

        _signpost = d.pop("signpost", UNSET)
        signpost: SignpostInfo | Unset
        if isinstance(_signpost, Unset):
            signpost = UNSET
        else:
            signpost = SignpostInfo.from_dict(_signpost)

        _intersection_name = d.pop("intersectionName", UNSET)
        intersection_name: list[LocalizedString] | Unset = UNSET
        if _intersection_name is not UNSET:
            intersection_name = []
            for intersection_name_item_data in _intersection_name:
                intersection_name_item = LocalizedString.from_dict(
                    intersection_name_item_data
                )

                intersection_name.append(intersection_name_item)

        _direction = d.pop("direction", UNSET)
        direction: TurnActionDirection | Unset
        if isinstance(_direction, Unset):
            direction = UNSET
        else:
            direction = TurnActionDirection(_direction)

        _severity = d.pop("severity", UNSET)
        severity: TurnActionSeverity | Unset
        if isinstance(_severity, Unset):
            severity = UNSET
        else:
            severity = TurnActionSeverity(_severity)

        turn_angle = d.pop("turnAngle", UNSET)

        simple_turn_action = cls(
            action=action,
            duration=duration,
            instruction=instruction,
            offset=offset,
            length=length,
            current_road=current_road,
            next_road=next_road,
            exit_sign=exit_sign,
            signpost=signpost,
            intersection_name=intersection_name,
            direction=direction,
            severity=severity,
            turn_angle=turn_angle,
        )

        simple_turn_action.additional_properties = d
        return simple_turn_action

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.departure import Departure
    from ..models.isoline import Isoline
    from ..models.isoline_response_notice import IsolineResponseNotice


T = TypeVar("T", bound="IsolineResponse")


@_attrs_define
class IsolineResponse:
    """
    Attributes:
        isolines (list[Isoline]): List of polygons calculated per specified range.
        notices (list[IsolineResponseNotice] | Unset): Contains a list of issues related to this isoline calculation.
            Please refer to the `code` attribute for possible values.
        departure (Departure | Unset): Describe a departure or arrival location and time.
        arrival (Departure | Unset): Describe a departure or arrival location and time.
    """

    isolines: list[Isoline]
    notices: list[IsolineResponseNotice] | Unset = UNSET
    departure: Departure | Unset = UNSET
    arrival: Departure | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        isolines = []
        for isolines_item_data in self.isolines:
            isolines_item = isolines_item_data.to_dict()
            isolines.append(isolines_item)

        notices: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.notices, Unset):
            notices = []
            for notices_item_data in self.notices:
                notices_item = notices_item_data.to_dict()
                notices.append(notices_item)

        departure: dict[str, Any] | Unset = UNSET
        if not isinstance(self.departure, Unset):
            departure = self.departure.to_dict()

        arrival: dict[str, Any] | Unset = UNSET
        if not isinstance(self.arrival, Unset):
            arrival = self.arrival.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "isolines": isolines,
        })
        if notices is not UNSET:
            field_dict["notices"] = notices
        if departure is not UNSET:
            field_dict["departure"] = departure
        if arrival is not UNSET:
            field_dict["arrival"] = arrival

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.departure import Departure
        from ..models.isoline import Isoline
        from ..models.isoline_response_notice import IsolineResponseNotice

        d = dict(src_dict)
        isolines = []
        _isolines = d.pop("isolines")
        for isolines_item_data in _isolines:
            isolines_item = Isoline.from_dict(isolines_item_data)

            isolines.append(isolines_item)

        _notices = d.pop("notices", UNSET)
        notices: list[IsolineResponseNotice] | Unset = UNSET
        if _notices is not UNSET:
            notices = []
            for notices_item_data in _notices:
                notices_item = IsolineResponseNotice.from_dict(notices_item_data)

                notices.append(notices_item)

        _departure = d.pop("departure", UNSET)
        departure: Departure | Unset
        if isinstance(_departure, Unset):
            departure = UNSET
        else:
            departure = Departure.from_dict(_departure)

        _arrival = d.pop("arrival", UNSET)
        arrival: Departure | Unset
        if isinstance(_arrival, Unset):
            arrival = UNSET
        else:
            arrival = Departure.from_dict(_arrival)

        isoline_response = cls(
            isolines=isolines,
            notices=notices,
            departure=departure,
            arrival=arrival,
        )

        isoline_response.additional_properties = d
        return isoline_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Phoneme")


@_attrs_define
class Phoneme:
    """
    Attributes:
        value (str): The actual phonetic transcription in the NT-SAMPA format.
        language (str | Unset): The [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) language code.
        preferred (bool | Unset): Whether or not it is the preferred phoneme.
    """

    value: str
    language: str | Unset = UNSET
    preferred: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        value = self.value

        language = self.language

        preferred = self.preferred

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "value": value,
        })
        if language is not UNSET:
            field_dict["language"] = language
        if preferred is not UNSET:
            field_dict["preferred"] = preferred

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        value = d.pop("value")

        language = d.pop("language", UNSET)

        preferred = d.pop("preferred", UNSET)

        phoneme = cls(
            value=value,
            language=language,
            preferred=preferred,
        )

        phoneme.additional_properties = d
        return phoneme

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error_response import ErrorResponse
from ...models.version_response import VersionResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    x_request_id: str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(x_request_id, Unset):
        headers["X-Request-ID"] = x_request_id

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/version",
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> ErrorResponse | VersionResponse | None:
    if response.status_code == 200:
        response_200 = VersionResponse.from_dict(response.json())

        return response_200

    if response.status_code == 500:
        response_500 = ErrorResponse.from_dict(response.json())

        return response_500

    if response.status_code == 502:
        response_502 = ErrorResponse.from_dict(response.json())

        return response_502

    if response.status_code == 503:
        response_503 = ErrorResponse.from_dict(response.json())

        return response_503

    if response.status_code == 504:
        response_504 = ErrorResponse.from_dict(response.json())

        return response_504

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[ErrorResponse | VersionResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    x_request_id: str | Unset = UNSET,
) -> Response[ErrorResponse | VersionResponse]:
    """Full version of the API

     Returns the version of the service

    Args:
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | VersionResponse]
    """

    kwargs = _get_kwargs(
        x_request_id=x_request_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    x_request_id: str | Unset = UNSET,
) -> ErrorResponse | VersionResponse | None:
    """Full version of the API

     Returns the version of the service

    Args:
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | VersionResponse
    """

    return sync_detailed(
        client=client,
        x_request_id=x_request_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    x_request_id: str | Unset = UNSET,
) -> Response[ErrorResponse | VersionResponse]:
    """Full version of the API

     Returns the version of the service

    Args:
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | VersionResponse]
    """

    kwargs = _get_kwargs(
        x_request_id=x_request_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    x_request_id: str | Unset = UNSET,
) -> ErrorResponse | VersionResponse | None:
    """Full version of the API

     Returns the version of the service

    Args:
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | VersionResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            x_request_id=x_request_id,
        )
    ).parsed



from enum import Enum


class EvChargingAttributesAccess(str, Enum):
    PRIVATE = "private"
    PUBLIC = "public"
    RESTRICTED = "restricted"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class FuelStationFuelTypesItem(str, Enum):
    BIODIESEL = "biodiesel"
    CNG = "cng"
    DIESEL = "diesel"
    DIESEL_WITH_ADDITIVES = "diesel_with_additives"
    E10 = "e10"
    E20 = "e20"
    E85 = "e85"
    ETHANOL = "ethanol"
    ETHANOL_WITH_ADDITIVES = "ethanol_with_additives"
    GASOLINE = "gasoline"
    HVO = "hvo"
    HYDROGEN = "hydrogen"
    LNG = "lng"
    LPG = "lpg"
    MIDGRADE = "midgrade"
    OCTANE_100 = "octane_100"
    OCTANE_87 = "octane_87"
    OCTANE_89 = "octane_89"
    OCTANE_90 = "octane_90"
    OCTANE_91 = "octane_91"
    OCTANE_92 = "octane_92"
    OCTANE_93 = "octane_93"
    OCTANE_95 = "octane_95"
    OCTANE_98 = "octane_98"
    PREMIUM = "premium"
    REGULAR = "regular"
    TRUCK_CNG = "truck_cng"
    TRUCK_DIESEL = "truck_diesel"
    TRUCK_HYDROGEN = "truck_hydrogen"
    TRUCK_LNG = "truck_lng"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.open_search_multi_reverse_geocode_error_result import (
        OpenSearchMultiReverseGeocodeErrorResult,
    )
    from ..models.open_search_multi_reverse_geocode_response_item import (
        OpenSearchMultiReverseGeocodeResponseItem,
    )


T = TypeVar("T", bound="OpenSearchMultiReverseGeocodeResponse")


@_attrs_define
class OpenSearchMultiReverseGeocodeResponse:
    """
    Attributes:
        results (list[OpenSearchMultiReverseGeocodeErrorResult | OpenSearchMultiReverseGeocodeResponseItem]): The
            results for the individual queries which either contain items or an error description
    """

    results: list[
        OpenSearchMultiReverseGeocodeErrorResult
        | OpenSearchMultiReverseGeocodeResponseItem
    ]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.open_search_multi_reverse_geocode_error_result import (
            OpenSearchMultiReverseGeocodeErrorResult,
        )

        results = []
        for results_item_data in self.results:
            results_item: dict[str, Any]
            if isinstance(results_item_data, OpenSearchMultiReverseGeocodeErrorResult):
                results_item = results_item_data.to_dict()
            else:
                results_item = results_item_data.to_dict()

            results.append(results_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "results": results,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.open_search_multi_reverse_geocode_error_result import (
            OpenSearchMultiReverseGeocodeErrorResult,
        )
        from ..models.open_search_multi_reverse_geocode_response_item import (
            OpenSearchMultiReverseGeocodeResponseItem,
        )

        d = dict(src_dict)
        results = []
        _results = d.pop("results")
        for results_item_data in _results:

            def _parse_results_item(
                data: object,
            ) -> (
                OpenSearchMultiReverseGeocodeErrorResult
                | OpenSearchMultiReverseGeocodeResponseItem
            ):
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    results_item_type_0 = (
                        OpenSearchMultiReverseGeocodeErrorResult.from_dict(data)
                    )

                    return results_item_type_0
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                results_item_type_1 = (
                    OpenSearchMultiReverseGeocodeResponseItem.from_dict(data)
                )

                return results_item_type_1

            results_item = _parse_results_item(results_item_data)

            results.append(results_item)

        open_search_multi_reverse_geocode_response = cls(
            results=results,
        )

        open_search_multi_reverse_geocode_response.additional_properties = d
        return open_search_multi_reverse_geocode_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Location")


@_attrs_define
class Location:
    """Location on the Earth

    Example:
        {'lat': 52.531677, 'lng': 13.381777}

    Attributes:
        lat (float): Location of a point on the Earth north or south of the equator in decimal degrees. Example:
            52.531677.
        lng (float): Location of a place on the Earth east or west of the prime meridian in decimal degrees. Example:
            13.381777.
        elv (float | Unset): Ellipsoid(geodetic) height in meters. Difference between the WGS84 ellipsoid and a point on
            the Earth’s surface.
            Note: Similar elevation can be obtained from a GPS receiver.
             Example: 512.5.
    """

    lat: float
    lng: float
    elv: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        lat = self.lat

        lng = self.lng

        elv = self.elv

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "lat": lat,
            "lng": lng,
        })
        if elv is not UNSET:
            field_dict["elv"] = elv

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        lat = d.pop("lat")

        lng = d.pop("lng")

        elv = d.pop("elv", UNSET)

        location = cls(
            lat=lat,
            lng=lng,
            elv=elv,
        )

        location.additional_properties = d
        return location

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="FarePassValidityPeriod")


@_attrs_define
class FarePassValidityPeriod:
    """Specifies a temporal validity period for a pass

    Attributes:
        period (str): Extensible enum: `annual` `extendedAnnual` `minutes` `days` `months` `...`
            Specifies one of the following validity periods:
              - `annual`: pass is valid from Jan 1 to Dec 31
              - `extendedAnnual`: pass is valid from Jan 1 to Jan 31 of the following year
              - `minutes`: pass is valid for a specified number of minutes See `unit`.
              - `days`: pass is valid for a specified number of days. See `unit`.
              - `months`: pass is valid for a specified number of months. See `unit`.
        count (int | Unset): Required if period is `minutes`, days` or `months`, it specifies how many of these units
            are covered by the pass.
    """

    period: str
    count: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        period = self.period

        count = self.count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "period": period,
        })
        if count is not UNSET:
            field_dict["count"] = count

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        period = d.pop("period")

        count = d.pop("count", UNSET)

        fare_pass_validity_period = cls(
            period=period,
            count=count,
        )

        fare_pass_validity_period.additional_properties = d
        return fare_pass_validity_period

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="MapView")


@_attrs_define
class MapView:
    """
    Attributes:
        west (float): Longitude of the western-side of the box. For example: "8.80068"
        south (float): Latitude of the southern-side of the box. For example: "52.19333"
        east (float): Longitude of the eastern-side of the box. For example: "8.8167"
        north (float): Latitude of the northern-side of the box. For example: "52.19555"
    """

    west: float
    south: float
    east: float
    north: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        west = self.west

        south = self.south

        east = self.east

        north = self.north

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "west": west,
            "south": south,
            "east": east,
            "north": north,
        })

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        west = d.pop("west")

        south = d.pop("south")

        east = d.pop("east")

        north = d.pop("north")

        map_view = cls(
            west=west,
            south=south,
            east=east,
            north=north,
        )

        map_view.additional_properties = d
        return map_view

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MaxSpeedOnSegmentPostInner")


@_attrs_define
class MaxSpeedOnSegmentPostInner:
    """
    Attributes:
        segment (str | Unset): Identifier of the segment with restrictions on maximum `baseSpeed`.

            Each entry has the following structure:
            `{segmentId}(#{direction})?`
        speed (float | Unset): Maximum `baseSpeed` on segment in m/s.
    """

    segment: str | Unset = UNSET
    speed: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        segment = self.segment

        speed = self.speed

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if segment is not UNSET:
            field_dict["segment"] = segment
        if speed is not UNSET:
            field_dict["speed"] = speed

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        segment = d.pop("segment", UNSET)

        speed = d.pop("speed", UNSET)

        max_speed_on_segment_post_inner = cls(
            segment=segment,
            speed=speed,
        )

        max_speed_on_segment_post_inner.additional_properties = d
        return max_speed_on_segment_post_inner

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.time_restricted_weekdays import TimeRestrictedWeekdays
from ..types import UNSET, Unset

T = TypeVar("T", bound="TimeRestrictedPrice")


@_attrs_define
class TimeRestrictedPrice:
    """
    Attributes:
        type_ (str): Type of price represented by this object. The API customer is responsible for correctly visualizing
            the pricing model. As it is possible to extend the supported price types in the future,
            the price information should be hidden when an unknown type is encountered.

            Available price types are:

              * `restricted` - A single price value valid for a specific time or duration
        currency (str): Local currency of the price compliant to ISO 4217
        value (float): The price value
        estimated (bool | Unset): Attribute value is `true` if the fare price is estimated, `false` if it is an exact
            value. Default: False.
        unit (int | Unset): Duration in seconds. Example: 198.
        days (list[TimeRestrictedWeekdays] | Unset): This price applies only for the selected days Example: ['sa',
            'su'].
        min_duration (int | Unset): Duration in seconds. Example: 198.
        max_duration (int | Unset): Duration in seconds. Example: 198.
        from_time (str | Unset): **RFC 3339**, section 5.6 as defined by `partial-time`. Example: 08:30:00.
        to_time (str | Unset): **RFC 3339**, section 5.6 as defined by `partial-time`. Example: 08:30:00.
    """

    type_: str
    currency: str
    value: float
    estimated: bool | Unset = False
    unit: int | Unset = UNSET
    days: list[TimeRestrictedWeekdays] | Unset = UNSET
    min_duration: int | Unset = UNSET
    max_duration: int | Unset = UNSET
    from_time: str | Unset = UNSET
    to_time: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        currency = self.currency

        value = self.value

        estimated = self.estimated

        unit = self.unit

        days: list[str] | Unset = UNSET
        if not isinstance(self.days, Unset):
            days = []
            for days_item_data in self.days:
                days_item = days_item_data.value
                days.append(days_item)

        min_duration = self.min_duration

        max_duration = self.max_duration

        from_time = self.from_time

        to_time = self.to_time

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "currency": currency,
            "value": value,
        })
        if estimated is not UNSET:
            field_dict["estimated"] = estimated
        if unit is not UNSET:
            field_dict["unit"] = unit
        if days is not UNSET:
            field_dict["days"] = days
        if min_duration is not UNSET:
            field_dict["minDuration"] = min_duration
        if max_duration is not UNSET:
            field_dict["maxDuration"] = max_duration
        if from_time is not UNSET:
            field_dict["fromTime"] = from_time
        if to_time is not UNSET:
            field_dict["toTime"] = to_time

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = d.pop("type")

        currency = d.pop("currency")

        value = d.pop("value")

        estimated = d.pop("estimated", UNSET)

        unit = d.pop("unit", UNSET)

        _days = d.pop("days", UNSET)
        days: list[TimeRestrictedWeekdays] | Unset = UNSET
        if _days is not UNSET:
            days = []
            for days_item_data in _days:
                days_item = TimeRestrictedWeekdays(days_item_data)

                days.append(days_item)

        min_duration = d.pop("minDuration", UNSET)

        max_duration = d.pop("maxDuration", UNSET)

        from_time = d.pop("fromTime", UNSET)

        to_time = d.pop("toTime", UNSET)

        time_restricted_price = cls(
            type_=type_,
            currency=currency,
            value=value,
            estimated=estimated,
            unit=unit,
            days=days,
            min_duration=min_duration,
            max_duration=max_duration,
            from_time=from_time,
            to_time=to_time,
        )

        time_restricted_price.additional_properties = d
        return time_restricted_price

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.notice_severity import NoticeSeverity
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.vehicle_restriction import VehicleRestriction
    from ..models.violated_charging_station_opening_hours import (
        ViolatedChargingStationOpeningHours,
    )
    from ..models.violated_transport_mode import ViolatedTransportMode
    from ..models.violated_truck_road_type import ViolatedTruckRoadType
    from ..models.violated_zone_reference import ViolatedZoneReference


T = TypeVar("T", bound="VehicleNotice")


@_attrs_define
class VehicleNotice:
    """A notice contains important notifications.

    Example:
        {'$ref': '#/components/examples/noticeWithRestrictionsExample'}

    Attributes:
        code (str): Extensible enum: `violatedAvoidControlledAccessHighway` `violatedAvoidTollRoad`
            `violatedAvoidTunnel` `violatedAvoidDirtRoad` `violatedBlockedRoad` `violatedStartDirection` `violatedCarpool`
            `violatedTurnRestriction` `violatedVehicleRestriction` `violatedZoneRestriction` `violatedAvoidDifficultTurns`
            `violatedAvoidUTurns` `violatedEmergencyGate` `violatedAvoidSeasonalClosure` `violatedAvoidTollTransponder`
            `violatedAvoidTruckRoadType` `seasonalClosure` `tollTransponder` `mlDurationUnavailable` `simplePolyline`
            `tollsDataUnavailable` `tollsDataTemporarilyUnavailable` `chargingStopNotNeeded`
            `violatedChargingStationOpeningHours` `violatedMinChargeAtDestination` `violatedMinChargeAtFirstChargingStation`
            `violatedMinChargeAtChargingStation` `outOfCharge` `currentWeightNegative` `currentWeightExceedsLimit`
            `travelTimeExceedsDriverWorkHours` `carpool` `turnRestriction` `vehicleRestriction` `zoneRestriction`
            `avoidTollRoad` `targetChargeNotAchievable` `violatedChargingStationTransportMode`
            `violatedChargingStationVehicleRestrictions` `...`
            Currently known codes (non-exhaustive: this list could be extended for new situations):

            | Code      | Description  | Severity |
            | --------- | ------- | ------- |
            | violatedAvoidControlledAccessHighway | Route did not manage to avoid user preference. | critical |
            | violatedAvoidTollRoad | Route did not manage to avoid user preference. | critical |
            | violatedAvoidTunnel | Route did not manage to avoid user preference. | critical |
            | violatedAvoidDirtRoad | Route did not manage to avoid user preference. | critical |
            | violatedBlockedRoad | Route uses roads that are blocked, due to traffic incidents, `avoid[areas]`, or
            `avoid[segments]`. | critical |
            | violatedStartDirection | The starting direction of the route differs from the value set in the `course`
            parameter of the waypoint. | critical |
            | violatedCarpool | Route did not manage to avoid user preference. | critical |
            | violatedTurnRestriction | Route requires performing a time-restricted maneuver at a time when it's prohibited.
            | critical |
            | violatedVehicleRestriction | Route uses a road which is forbidden for the given vehicle profile. | critical |
            | violatedZoneRestriction | Route uses a road in a user-specified avoidance zone, such as the London Low
            Emission Zone. | critical |
            | violatedAvoidDifficultTurns | Route did not manage to avoid user preference.| critical |
            | violatedAvoidUTurns | Route did not manage to avoid user preference. | critical |
            | violatedEmergencyGate | Route goes through an emergency gate. | critical |
            | violatedAvoidSeasonalClosure | Route did not manage to avoid a seasonal closure.| critical |
            | violatedAvoidTollTransponder | Route did not manage to avoid a toll booth that requires a transponder. |
            critical |
            | violatedAvoidTruckRoadType | Route did not manage to avoid truck road type. | critical |
            | seasonalClosure | Route goes through a seasonal closure. | info |
            | tollTransponder | Route goes through a toll booth that requires a transponder. | info |
            | mlDurationUnavailable | Machine learning duration was requested but is not available for this section. | info
            |
            | simplePolyline | An accurate polyline is not available for this section. The returned polyline has been
            generated from departure and arrival places. | info |
            | tollsDataUnavailable | Tolls data was requested but could not be calculated for this section. | info |
            | tollsDataTemporarilyUnavailable | Tolls data was requested but is temporarily unavailable. | info |
            | chargingStopNotNeeded | A charging stop was planned at the destination of this section, but it is no longer
            needed (getRoutesByHandle requests only). | info |
            | violatedChargingStationOpeningHours | A charging stop was planned at the destination of this section, but the
            `postActions` would not be completed within the opening hours. | critical |
            | violatedMinChargeAtDestination | The `arrival.charge` at the destination is lower than the
            `minChargeAtDestination`. | info |
            | violatedMinChargeAtFirstChargingStation | The `arrival.charge` on the first charging stop is lower than the
            `minChargeAtFirstChargingStation`. | info |
            | violatedMinChargeAtChargingStation | The `arrival.charge` on a charging station stop is lower than the
            `minChargeAtChargingStation`. This can be on the first charging stop if `minChargeAtFirstChargingStation` is not
            specified. | info |
            | outOfCharge | The charge of the EV drops below 0 along the section. | info |
            | currentWeightNegative | `vehicle[currentWeight]` is negative after applying `currentWeightChange` at a via
            waypoint. | critical |
            | currentWeightExceedsLimit | `vehicle[currentWeight]` exceeds limit for given `transportMode`. | critical |
            | travelTimeExceedsDriverWorkHours | The route travelTime exceeds drive time sequence provided in
            `driver[schedule]` parameter. | critical |
            | targetChargeNotAchievable | `ev[MaxChargingDuration]` too short to achieve required charging level. | info |
            | violatedChargingStationTransportMode | Provided `transportMode` is not permitted at the charging station |
            critical |
            | violatedChargingStationVehicleRestrictions | One or more vehicle attributes exceed the permitted vehicle
            limits of the charging station | critical |


            The codes listed below can appear as info notices when the request contains
            `return=potentialTimeDependentViolations`.
            The resulting info notices are not violations on the current route but could be violations if the arrival time
            at these restrictions differs from the calculated time.
            For example, an info notice for a `vehicleRestriction` could indicate that while your vehicle is not currently
            violating any restriction.
            However, if the journey is delayed by an hour the vehicle would not be allowed on part of the route because
            there is a time-dependent restriction active at that time.

            Codes for potential time dependent restriction violations are (non-exhaustive: can be extended for new
            situations):

            | Code      | Description  | Severity |
            | --------- | ------- | ------- |
            | carpool | Route potentially uses a carpool only road when not allowed. | info |
            | turnRestriction | Route potentially uses a time-restricted turn when not allowed. | info |
            | vehicleRestriction | Route potentially uses a road that is forbidden for the given vehicle profile. | info |
            | zoneRestriction | Route uses a road that is potentially part of restricted routing zones. | info |
            | avoidTollRoad | Route uses a road that potentially is a toll road. **Note:** This is not currently supported
            and will not be returned, but the enum value is retained for backwards-compatibility purposes. | info |
             Example: violatedAvoidTollRoad.
        title (str | Unset): Human-readable notice description. Example: Violated avoid toll road.
        severity (NoticeSeverity | Unset): Describes the impact a notice has on the resource to which the notice is
            attached.
            * critical - The notice must not be ignored, even if the type of notice is not known to the user. Any associated
            resource (e.g., route section) must not be used without further evaluation.
            * info - The notice is for informative purposes, but does not affect usability of the route.
        details (list[VehicleRestriction | ViolatedChargingStationOpeningHours | ViolatedTransportMode |
            ViolatedTruckRoadType | ViolatedZoneReference] | Unset): Additional details about the notice
    """

    code: str
    title: str | Unset = UNSET
    severity: NoticeSeverity | Unset = UNSET
    details: (
        list[
            VehicleRestriction
            | ViolatedChargingStationOpeningHours
            | ViolatedTransportMode
            | ViolatedTruckRoadType
            | ViolatedZoneReference
        ]
        | Unset
    ) = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.vehicle_restriction import VehicleRestriction
        from ..models.violated_transport_mode import ViolatedTransportMode
        from ..models.violated_truck_road_type import ViolatedTruckRoadType
        from ..models.violated_zone_reference import ViolatedZoneReference

        code = self.code

        title = self.title

        severity: str | Unset = UNSET
        if not isinstance(self.severity, Unset):
            severity = self.severity.value

        details: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.details, Unset):
            details = []
            for details_item_data in self.details:
                details_item: dict[str, Any]
                if isinstance(details_item_data, VehicleRestriction):
                    details_item = details_item_data.to_dict()
                elif isinstance(details_item_data, ViolatedTransportMode):
                    details_item = details_item_data.to_dict()
                elif isinstance(details_item_data, ViolatedTruckRoadType):
                    details_item = details_item_data.to_dict()
                elif isinstance(details_item_data, ViolatedZoneReference):
                    details_item = details_item_data.to_dict()
                else:
                    details_item = details_item_data.to_dict()

                details.append(details_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "code": code,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if severity is not UNSET:
            field_dict["severity"] = severity
        if details is not UNSET:
            field_dict["details"] = details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.vehicle_restriction import VehicleRestriction
        from ..models.violated_charging_station_opening_hours import (
            ViolatedChargingStationOpeningHours,
        )
        from ..models.violated_transport_mode import ViolatedTransportMode
        from ..models.violated_truck_road_type import ViolatedTruckRoadType
        from ..models.violated_zone_reference import ViolatedZoneReference

        d = dict(src_dict)
        code = d.pop("code")

        title = d.pop("title", UNSET)

        _severity = d.pop("severity", UNSET)
        severity: NoticeSeverity | Unset
        if isinstance(_severity, Unset):
            severity = UNSET
        else:
            severity = NoticeSeverity(_severity)

        _details = d.pop("details", UNSET)
        details: (
            list[
                VehicleRestriction
                | ViolatedChargingStationOpeningHours
                | ViolatedTransportMode
                | ViolatedTruckRoadType
                | ViolatedZoneReference
            ]
            | Unset
        ) = UNSET
        if _details is not UNSET:
            details = []
            for details_item_data in _details:

                def _parse_details_item(
                    data: object,
                ) -> (
                    VehicleRestriction
                    | ViolatedChargingStationOpeningHours
                    | ViolatedTransportMode
                    | ViolatedTruckRoadType
                    | ViolatedZoneReference
                ):
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_notice_detail_type_0 = (
                            VehicleRestriction.from_dict(data)
                        )

                        return componentsschemas_vehicle_notice_detail_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_notice_detail_type_1 = (
                            ViolatedTransportMode.from_dict(data)
                        )

                        return componentsschemas_vehicle_notice_detail_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_notice_detail_type_2 = (
                            ViolatedTruckRoadType.from_dict(data)
                        )

                        return componentsschemas_vehicle_notice_detail_type_2
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_vehicle_notice_detail_type_3 = (
                            ViolatedZoneReference.from_dict(data)
                        )

                        return componentsschemas_vehicle_notice_detail_type_3
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_vehicle_notice_detail_type_4 = (
                        ViolatedChargingStationOpeningHours.from_dict(data)
                    )

                    return componentsschemas_vehicle_notice_detail_type_4

                details_item = _parse_details_item(details_item_data)

                details.append(details_item)

        vehicle_notice = cls(
            code=code,
            title=title,
            severity=severity,
            details=details,
        )

        vehicle_notice.additional_properties = d
        return vehicle_notice

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import argparse
import datetime
from urllib.parse import urljoin

from . import _consts, _util
from ._apiclient.geocoder import AuthenticatedClient as GeocodingClient
from ._apiclient.isolines import AuthenticatedClient as IsolinesClient
from ._apiclient.old import AuthenticatedClient as V1Client
from ._apiclient.router import AuthenticatedClient as RoutingClient
from ._errors import CasaGeoError
from ._types import CasaGeoResult


class AccountResult(CasaGeoResult):
    """
    Represents the result of an account lookup operation.
    """

    def username(self) -> str:
        """Return the username associated with the account."""
        return self._data.get("user", "")

    def account_type(self) -> str:
        """Return a string describing the type of the account."""
        return self._data.get("note", "")

    def credits(self) -> int:
        """Return the number of available credits."""
        return self._data.get("credits", 0)

    def expires(self) -> datetime.datetime | None:
        """Return the expiration date and time of the account."""
        try:
            return datetime.datetime.fromisoformat(self._data["expires"])
        except (KeyError, ValueError):
            return None


class CasaGeoClient:
    """
    The casaGeo API client.

    The ``preferred_*`` attributes are used as default values by
    CasaGeoCoder, CasaGeoSpatial, etc. when making API requests.

    Attributes:
        preferred_language:
            The preferred language for responses. This must be a valid IETF
            BCP47 language tag, such as "en-US", or a comma-separated list of
            such tags in order of preference.

        preferred_political_view:
            The preferred political view for responses. This must be a valid ISO
            3166-1 alpha-3 country code.

        preferred_unit_system:
            The preferred unit system for responses, either ``"metric"`` or
            ``"imperial"``.

    Parameters:
        key: Your casaGeo API license key.
    """

    def __init__(
        self,
        key: str,
        *,
        preferred_language: str | None = None,
        preferred_political_view: str | None = None,
        preferred_unit_system: str | None = None,
    ):
        server = _consts.SERVER
        server_api_v1 = urljoin(server, "api/v1/")

        self._v1_client = V1Client(base_url=server, token=key, prefix="")
        self._geocoding_client = GeocodingClient(
            base_url=server_api_v1, token=key, prefix=""
        )
        self._isolines_client = IsolinesClient(
            base_url=server_api_v1, token=key, prefix=""
        )
        self._routing_client = RoutingClient(
            base_url=server_api_v1, token=key, prefix=""
        )

        self.preferred_language = preferred_language
        self.preferred_political_view = preferred_political_view
        self.preferred_unit_system = preferred_unit_system

    @AccountResult.wrap_errors()
    def account_info(self) -> AccountResult:
        """
        Query the server for account information.

        Returns:
            AccountResult: An AccountResult instance.
        """
        from ._apiclient.old.api.v1 import v1_accountinfo_retrieve

        return AccountResult.from_response(
            v1_accountinfo_retrieve.sync_detailed(client=self._v1_client)
        )


def _main(args: argparse.Namespace):
    cga = CasaGeoClient(args.key, preferred_language=args.lang)
    match args.command:
        case "account-info":
            return cga.account_info().json()
        case _:
            raise CasaGeoError(f"Unknown command: {args.command}")


if __name__ == "__main__":
    _util.cli_main(_consts.get_client_parser(), _main)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.notice_severity import NoticeSeverity
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.vehicle_restriction import VehicleRestriction
    from ..models.violated_transport_mode import ViolatedTransportMode


T = TypeVar("T", bound="TransitNotice")


@_attrs_define
class TransitNotice:
    """A notice contains important notifications.

    Attributes:
        code (str): Extensible enum: `noSchedule` `noIntermediate` `unwantedMode` `scheduledTimes` `simplePolyline`
            `violatedAvoidFerry` `violatedAvoidRailFerry` `seasonalClosure` `currentWeightNegative`
            `travelTimeExceedsDriverWorkHours` `vehicleRestriction` `...`
            Currently known codes (non-exhaustive: this list could be extended for new situations):

            | Code      | Description  | Severity |
            | --------- | ------- | ------- |
            | noSchedule | A timetable schedule is not available for the transit line in this section, and only the run
            frequency is available. As a result, departure/arrival times are approximated | info |
            | noIntermediate | Information about intermediate stops is not available for this transit line | info |
            | unwantedMode | This section contains a transport mode that was explicitly disabled. Mode filtering is not
            available in this area | info |
            | scheduledTimes | The times returned on this section are the scheduled times even though delay information are
            available | info |
            | simplePolyline | An accurate polyline is not available for this section. The returned polyline has been
            generated from departure and arrival places | info |
            | violatedAvoidFerry | Route did not manage to avoid user preference | critical |
            | violatedAvoidRailFerry | Route did not manage to avoid user preference | critical |
            | seasonalClosure | Route goes through seasonal closure | info |
            | currentWeightNegative | `vehicle[currentWeight]` is negative after applying `currentWeightChange` at a via
            waypoint. | critical |
            | travelTimeExceedsDriverWorkHours | The route travelTime exceeds drive time sequence provided in
            `driver[schedule]` parameter | critical |


            The codes listed below can appear as info notices when the request contains
            `return=potentialTimeDependentViolations`.
            The resulting info notices are not violations on the current route but could be violations if the arrival time
            at these restrictions differs from the calculated time.
            For example, an info notice for a `vehicleRestriction` could indicate that your vehicle is not currently
            violating any restriction but,
            If the journey is delayed by an hour, the vehicle would not be allowed on part of the route because there is a
            time-dependent restriction active at that time.

            Codes for potential time-dependent restriction violations are (non-exhaustive: can be extended for new
            situations):

            | Code      | Description  | Severity |
            | --------- | ------- | ------- |
            | vehicleRestriction | Route potentially uses a road that is forbidden for the given vehicle profile | info |
             Example: noSchedule.
        title (str | Unset): Human-readable notice description. Example: No schedule.
        severity (NoticeSeverity | Unset): Describes the impact a notice has on the resource to which the notice is
            attached.
            * critical - The notice must not be ignored, even if the type of notice is not known to the user. Any associated
            resource (e.g., route section) must not be used without further evaluation.
            * info - The notice is for informative purposes, but does not affect usability of the route.
        details (list[VehicleRestriction | ViolatedTransportMode] | Unset): Additional details about the notice
    """

    code: str
    title: str | Unset = UNSET
    severity: NoticeSeverity | Unset = UNSET
    details: list[VehicleRestriction | ViolatedTransportMode] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.vehicle_restriction import VehicleRestriction

        code = self.code

        title = self.title

        severity: str | Unset = UNSET
        if not isinstance(self.severity, Unset):
            severity = self.severity.value

        details: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.details, Unset):
            details = []
            for details_item_data in self.details:
                details_item: dict[str, Any]
                if isinstance(details_item_data, VehicleRestriction):
                    details_item = details_item_data.to_dict()
                else:
                    details_item = details_item_data.to_dict()

                details.append(details_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "code": code,
        })
        if title is not UNSET:
            field_dict["title"] = title
        if severity is not UNSET:
            field_dict["severity"] = severity
        if details is not UNSET:
            field_dict["details"] = details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.vehicle_restriction import VehicleRestriction
        from ..models.violated_transport_mode import ViolatedTransportMode

        d = dict(src_dict)
        code = d.pop("code")

        title = d.pop("title", UNSET)

        _severity = d.pop("severity", UNSET)
        severity: NoticeSeverity | Unset
        if isinstance(_severity, Unset):
            severity = UNSET
        else:
            severity = NoticeSeverity(_severity)

        _details = d.pop("details", UNSET)
        details: list[VehicleRestriction | ViolatedTransportMode] | Unset = UNSET
        if _details is not UNSET:
            details = []
            for details_item_data in _details:

                def _parse_details_item(
                    data: object,
                ) -> VehicleRestriction | ViolatedTransportMode:
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_transit_notice_detail_type_0 = (
                            VehicleRestriction.from_dict(data)
                        )

                        return componentsschemas_transit_notice_detail_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_transit_notice_detail_type_1 = (
                        ViolatedTransportMode.from_dict(data)
                    )

                    return componentsschemas_transit_notice_detail_type_1

                details_item = _parse_details_item(details_item_data)

                details.append(details_item)

        transit_notice = cls(
            code=code,
            title=title,
            severity=severity,
            details=details,
        )

        transit_notice.additional_properties = d
        return transit_notice

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error_response import ErrorResponse
from ...models.ev_station import EvStation
from ...models.fuel_station import FuelStation
from ...models.get_discover_mobility_mode import GetDiscoverMobilityMode
from ...models.get_discover_ranking import GetDiscoverRanking
from ...models.get_discover_show_item import GetDiscoverShowItem
from ...models.get_discover_with_item import GetDiscoverWithItem
from ...models.open_search_search_response import OpenSearchSearchResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    q: str,
    at: str | Unset = UNSET,
    fuel_station: FuelStation | Unset = UNSET,
    in_: str | Unset = UNSET,
    mobility_mode: GetDiscoverMobilityMode | Unset = UNSET,
    ranking: GetDiscoverRanking | Unset = UNSET,
    route: str | Unset = UNSET,
    with_: list[GetDiscoverWithItem] | Unset = UNSET,
    ev_station: EvStation | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 20,
    offset: int | Unset = UNSET,
    political_view: str | Unset = UNSET,
    show: list[GetDiscoverShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(x_request_id, Unset):
        headers["X-Request-ID"] = x_request_id

    params: dict[str, Any] = {}

    params["q"] = q

    params["at"] = at

    json_fuel_station: dict[str, Any] | Unset = UNSET
    if not isinstance(fuel_station, Unset):
        json_fuel_station = fuel_station.to_dict()
    if not isinstance(json_fuel_station, Unset):
        params.update((f"fuelStation[{k}]", v) for k, v in json_fuel_station.items())

    params["in"] = in_

    json_mobility_mode: str | Unset = UNSET
    if not isinstance(mobility_mode, Unset):
        json_mobility_mode = mobility_mode.value

    params["mobilityMode"] = json_mobility_mode

    json_ranking: str | Unset = UNSET
    if not isinstance(ranking, Unset):
        json_ranking = ranking.value

    params["ranking"] = json_ranking

    params["route"] = route

    json_with_: list[str] | Unset = UNSET
    if not isinstance(with_, Unset):
        json_with_ = []
        for with_item_data in with_:
            with_item = with_item_data.value
            json_with_.append(with_item)

    params["with"] = json_with_

    json_ev_station: dict[str, Any] | Unset = UNSET
    if not isinstance(ev_station, Unset):
        json_ev_station = ev_station.to_dict()
    if not isinstance(json_ev_station, Unset):
        params.update((f"evStation[{k}]", v) for k, v in json_ev_station.items())

    json_lang: list[str] | Unset = UNSET
    if not isinstance(lang, Unset):
        json_lang = lang

    params["lang"] = json_lang

    params["limit"] = limit

    params["offset"] = offset

    params["politicalView"] = political_view

    json_show: list[str] | Unset = UNSET
    if not isinstance(show, Unset):
        json_show = []
        for show_item_data in show:
            show_item = show_item_data.value
            json_show.append(show_item)

    params["show"] = json_show

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/discover",
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> ErrorResponse | OpenSearchSearchResponse | None:
    if response.status_code == 200:
        response_200 = OpenSearchSearchResponse.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = ErrorResponse.from_dict(response.json())

        return response_400

    if response.status_code == 405:
        response_405 = ErrorResponse.from_dict(response.json())

        return response_405

    if response.status_code == 429:
        response_429 = ErrorResponse.from_dict(response.json())

        return response_429

    if response.status_code == 503:
        response_503 = ErrorResponse.from_dict(response.json())

        return response_503

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[ErrorResponse | OpenSearchSearchResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    q: str,
    at: str | Unset = UNSET,
    fuel_station: FuelStation | Unset = UNSET,
    in_: str | Unset = UNSET,
    mobility_mode: GetDiscoverMobilityMode | Unset = UNSET,
    ranking: GetDiscoverRanking | Unset = UNSET,
    route: str | Unset = UNSET,
    with_: list[GetDiscoverWithItem] | Unset = UNSET,
    ev_station: EvStation | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 20,
    offset: int | Unset = UNSET,
    political_view: str | Unset = UNSET,
    show: list[GetDiscoverShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[ErrorResponse | OpenSearchSearchResponse]:
    """Discover

     This endpoint processes a free-form text query for an address or place, and returns results in order
    of relevance.

    Args:
        q (str):  Example: Eismieze Berlin.
        at (str | Unset):  Example: 52.5308,13.3856.
        fuel_station (FuelStation | Unset):
        in_ (str | Unset):
        mobility_mode (GetDiscoverMobilityMode | Unset):
        ranking (GetDiscoverRanking | Unset):
        route (str | Unset):
        with_ (list[GetDiscoverWithItem] | Unset):
        ev_station (EvStation | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 20.
        offset (int | Unset):
        political_view (str | Unset):
        show (list[GetDiscoverShowItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | OpenSearchSearchResponse]
    """

    kwargs = _get_kwargs(
        q=q,
        at=at,
        fuel_station=fuel_station,
        in_=in_,
        mobility_mode=mobility_mode,
        ranking=ranking,
        route=route,
        with_=with_,
        ev_station=ev_station,
        lang=lang,
        limit=limit,
        offset=offset,
        political_view=political_view,
        show=show,
        x_request_id=x_request_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    q: str,
    at: str | Unset = UNSET,
    fuel_station: FuelStation | Unset = UNSET,
    in_: str | Unset = UNSET,
    mobility_mode: GetDiscoverMobilityMode | Unset = UNSET,
    ranking: GetDiscoverRanking | Unset = UNSET,
    route: str | Unset = UNSET,
    with_: list[GetDiscoverWithItem] | Unset = UNSET,
    ev_station: EvStation | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 20,
    offset: int | Unset = UNSET,
    political_view: str | Unset = UNSET,
    show: list[GetDiscoverShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> ErrorResponse | OpenSearchSearchResponse | None:
    """Discover

     This endpoint processes a free-form text query for an address or place, and returns results in order
    of relevance.

    Args:
        q (str):  Example: Eismieze Berlin.
        at (str | Unset):  Example: 52.5308,13.3856.
        fuel_station (FuelStation | Unset):
        in_ (str | Unset):
        mobility_mode (GetDiscoverMobilityMode | Unset):
        ranking (GetDiscoverRanking | Unset):
        route (str | Unset):
        with_ (list[GetDiscoverWithItem] | Unset):
        ev_station (EvStation | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 20.
        offset (int | Unset):
        political_view (str | Unset):
        show (list[GetDiscoverShowItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | OpenSearchSearchResponse
    """

    return sync_detailed(
        client=client,
        q=q,
        at=at,
        fuel_station=fuel_station,
        in_=in_,
        mobility_mode=mobility_mode,
        ranking=ranking,
        route=route,
        with_=with_,
        ev_station=ev_station,
        lang=lang,
        limit=limit,
        offset=offset,
        political_view=political_view,
        show=show,
        x_request_id=x_request_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    q: str,
    at: str | Unset = UNSET,
    fuel_station: FuelStation | Unset = UNSET,
    in_: str | Unset = UNSET,
    mobility_mode: GetDiscoverMobilityMode | Unset = UNSET,
    ranking: GetDiscoverRanking | Unset = UNSET,
    route: str | Unset = UNSET,
    with_: list[GetDiscoverWithItem] | Unset = UNSET,
    ev_station: EvStation | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 20,
    offset: int | Unset = UNSET,
    political_view: str | Unset = UNSET,
    show: list[GetDiscoverShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[ErrorResponse | OpenSearchSearchResponse]:
    """Discover

     This endpoint processes a free-form text query for an address or place, and returns results in order
    of relevance.

    Args:
        q (str):  Example: Eismieze Berlin.
        at (str | Unset):  Example: 52.5308,13.3856.
        fuel_station (FuelStation | Unset):
        in_ (str | Unset):
        mobility_mode (GetDiscoverMobilityMode | Unset):
        ranking (GetDiscoverRanking | Unset):
        route (str | Unset):
        with_ (list[GetDiscoverWithItem] | Unset):
        ev_station (EvStation | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 20.
        offset (int | Unset):
        political_view (str | Unset):
        show (list[GetDiscoverShowItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | OpenSearchSearchResponse]
    """

    kwargs = _get_kwargs(
        q=q,
        at=at,
        fuel_station=fuel_station,
        in_=in_,
        mobility_mode=mobility_mode,
        ranking=ranking,
        route=route,
        with_=with_,
        ev_station=ev_station,
        lang=lang,
        limit=limit,
        offset=offset,
        political_view=political_view,
        show=show,
        x_request_id=x_request_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    q: str,
    at: str | Unset = UNSET,
    fuel_station: FuelStation | Unset = UNSET,
    in_: str | Unset = UNSET,
    mobility_mode: GetDiscoverMobilityMode | Unset = UNSET,
    ranking: GetDiscoverRanking | Unset = UNSET,
    route: str | Unset = UNSET,
    with_: list[GetDiscoverWithItem] | Unset = UNSET,
    ev_station: EvStation | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 20,
    offset: int | Unset = UNSET,
    political_view: str | Unset = UNSET,
    show: list[GetDiscoverShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> ErrorResponse | OpenSearchSearchResponse | None:
    """Discover

     This endpoint processes a free-form text query for an address or place, and returns results in order
    of relevance.

    Args:
        q (str):  Example: Eismieze Berlin.
        at (str | Unset):  Example: 52.5308,13.3856.
        fuel_station (FuelStation | Unset):
        in_ (str | Unset):
        mobility_mode (GetDiscoverMobilityMode | Unset):
        ranking (GetDiscoverRanking | Unset):
        route (str | Unset):
        with_ (list[GetDiscoverWithItem] | Unset):
        ev_station (EvStation | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 20.
        offset (int | Unset):
        political_view (str | Unset):
        show (list[GetDiscoverShowItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | OpenSearchSearchResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            q=q,
            at=at,
            fuel_station=fuel_station,
            in_=in_,
            mobility_mode=mobility_mode,
            ranking=ranking,
            route=route,
            with_=with_,
            ev_station=ev_station,
            lang=lang,
            limit=limit,
            offset=offset,
            political_view=political_view,
            show=show,
            x_request_id=x_request_id,
        )
    ).parsed



"""Contains all the data models used in inputs/outputs"""

from .allow import Allow
from .auth_error_response_schema import AuthErrorResponseSchema
from .avoid import Avoid
from .avoid_post import AvoidPost
from .base_ev_empirical_model import BaseEVEmpiricalModel
from .base_ev_physical_model import BaseEVPhysicalModel
from .base_notice_detail import BaseNoticeDetail
from .base_place import BasePlace
from .base_place_side_of_street import BasePlaceSideOfStreet
from .bounding_box_area import BoundingBoxArea
from .bounding_box_area_with_exceptions import BoundingBoxAreaWithExceptions
from .calculate_isolines_optimize_for import CalculateIsolinesOptimizeFor
from .calculate_isolines_post_optimize_for import CalculateIsolinesPostOptimizeFor
from .calculate_isolines_post_parameters import CalculateIsolinesPostParameters
from .connection import Connection
from .coordinate import Coordinate
from .corridor_area import CorridorArea
from .corridor_area_with_exceptions import CorridorAreaWithExceptions
from .data_version import DataVersion
from .departure import Departure
from .encoded_corridor_area import EncodedCorridorArea
from .encoded_corridor_area_with_exceptions import EncodedCorridorAreaWithExceptions
from .encoded_polygon_area import EncodedPolygonArea
from .encoded_polygon_area_with_exceptions import EncodedPolygonAreaWithExceptions
from .error_response import ErrorResponse
from .exclude import Exclude
from .exclude_post import ExcludePost
from .fuel import Fuel
from .health_response_fail_schema import HealthResponseFailSchema
from .health_response_fail_schema_status import HealthResponseFailSchemaStatus
from .health_response_ok_schema import HealthResponseOKSchema
from .health_response_ok_schema_status import HealthResponseOKSchemaStatus
from .isoline import Isoline
from .isoline_error_response import IsolineErrorResponse
from .isoline_response import IsolineResponse
from .isoline_response_notice import IsolineResponseNotice
from .location import Location
from .max_speed_on_segment_post_inner import MaxSpeedOnSegmentPostInner
from .not_allowed import NotAllowed
from .notice_severity import NoticeSeverity
from .polygon import Polygon
from .polygon_area import PolygonArea
from .polygon_area_with_exceptions import PolygonAreaWithExceptions
from .range_ import Range
from .response_range import ResponseRange
from .router_mode import RouterMode
from .routing_mode import RoutingMode
from .server_internal import ServerInternal
from .shape import Shape
from .taxi import Taxi
from .traffic import Traffic
from .traffic_mode import TrafficMode
from .truck import Truck
from .truck_category import TruckCategory
from .truck_engine_type import TruckEngineType
from .truck_type_with_default import TruckTypeWithDefault
from .tunnel_category import TunnelCategory
from .vehicle import Vehicle
from .vehicle_category import VehicleCategory
from .vehicle_engine_type import VehicleEngineType
from .vehicle_type import VehicleType
from .version_response import VersionResponse

__all__ = (
    "Allow",
    "AuthErrorResponseSchema",
    "Avoid",
    "AvoidPost",
    "BaseEVEmpiricalModel",
    "BaseEVPhysicalModel",
    "BaseNoticeDetail",
    "BasePlace",
    "BasePlaceSideOfStreet",
    "BoundingBoxArea",
    "BoundingBoxAreaWithExceptions",
    "CalculateIsolinesOptimizeFor",
    "CalculateIsolinesPostOptimizeFor",
    "CalculateIsolinesPostParameters",
    "Connection",
    "Coordinate",
    "CorridorArea",
    "CorridorAreaWithExceptions",
    "DataVersion",
    "Departure",
    "EncodedCorridorArea",
    "EncodedCorridorAreaWithExceptions",
    "EncodedPolygonArea",
    "EncodedPolygonAreaWithExceptions",
    "ErrorResponse",
    "Exclude",
    "ExcludePost",
    "Fuel",
    "HealthResponseFailSchema",
    "HealthResponseFailSchemaStatus",
    "HealthResponseOKSchema",
    "HealthResponseOKSchemaStatus",
    "Isoline",
    "IsolineErrorResponse",
    "IsolineResponse",
    "IsolineResponseNotice",
    "Location",
    "MaxSpeedOnSegmentPostInner",
    "NotAllowed",
    "NoticeSeverity",
    "Polygon",
    "PolygonArea",
    "PolygonAreaWithExceptions",
    "Range",
    "ResponseRange",
    "RouterMode",
    "RoutingMode",
    "ServerInternal",
    "Shape",
    "Taxi",
    "Traffic",
    "TrafficMode",
    "Truck",
    "TruckCategory",
    "TruckEngineType",
    "TruckTypeWithDefault",
    "TunnelCategory",
    "Vehicle",
    "VehicleCategory",
    "VehicleEngineType",
    "VehicleType",
    "VersionResponse",
)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.fare_pass import FarePass
    from ..models.range_price import RangePrice
    from ..models.single_price import SinglePrice
    from ..models.transponder_system import TransponderSystem


T = TypeVar("T", bound="Fare")


@_attrs_define
class Fare:
    """ "`Fare` type contains information about a single fare or ticket needed for this section of the route."

    **NOTE**: Fares information support will be available soon.

        Attributes:
            id (str): Unique Fare id. Used to deduplicate fares that apply to multiple sections Example:
                carrideco-1753076078174759852-1749341260512683512.
            name (str): Name of a fare Example: Carride Company.
            price (RangePrice | SinglePrice): Price of a fare
            converted_price (RangePrice | SinglePrice | Unset): Price of a fare
            reason (str | Unset): Extensible enum: `ride` `parking` `toll` `...`
                Reason for the cost described in this `Fare` element.
                 Default: 'ride'.
            payment_methods (list[str] | Unset): Specifies the payment methods for which this fare is valid.
            pass_ (FarePass | Unset): Specifies whether this `Fare` is a multi-travel pass, and its characteristics
            applicable_times (str | Unset): Specifies date and time period during which the restriction applies. Value is a
                string in the Time
                Domain format. Time Domain is part of the GDF (Geographic Data Files) specification, which is an ISO standard.
                Current standard is GDF 5.1 which is [ISO 20524-1:2020](https://www.iso.org/standard/68244.html).

                For a detailed description of the Time Domain specification and usage in routing services, please refer to
                the documentation available in the [Time Domain](https://www.here.com/docs/bundle/routing-api-developer-
                guide-v8/page/concepts/time-domain.html) page of the Developer Guide.
                 Example: -(d1){w1}(d3){d1}.
            transponders (list[TransponderSystem] | Unset): List of transponder systems for which this fare is applicable.
                Only available for transponder payment method
    """

    id: str
    name: str
    price: RangePrice | SinglePrice
    converted_price: RangePrice | SinglePrice | Unset = UNSET
    reason: str | Unset = "ride"
    payment_methods: list[str] | Unset = UNSET
    pass_: FarePass | Unset = UNSET
    applicable_times: str | Unset = UNSET
    transponders: list[TransponderSystem] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.single_price import SinglePrice

        id = self.id

        name = self.name

        price: dict[str, Any]
        if isinstance(self.price, SinglePrice):
            price = self.price.to_dict()
        else:
            price = self.price.to_dict()

        converted_price: dict[str, Any] | Unset
        if isinstance(self.converted_price, Unset):
            converted_price = UNSET
        elif isinstance(self.converted_price, SinglePrice):
            converted_price = self.converted_price.to_dict()
        else:
            converted_price = self.converted_price.to_dict()

        reason = self.reason

        payment_methods: list[str] | Unset = UNSET
        if not isinstance(self.payment_methods, Unset):
            payment_methods = self.payment_methods

        pass_: dict[str, Any] | Unset = UNSET
        if not isinstance(self.pass_, Unset):
            pass_ = self.pass_.to_dict()

        applicable_times = self.applicable_times

        transponders: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.transponders, Unset):
            transponders = []
            for transponders_item_data in self.transponders:
                transponders_item = transponders_item_data.to_dict()
                transponders.append(transponders_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "name": name,
            "price": price,
        })
        if converted_price is not UNSET:
            field_dict["convertedPrice"] = converted_price
        if reason is not UNSET:
            field_dict["reason"] = reason
        if payment_methods is not UNSET:
            field_dict["paymentMethods"] = payment_methods
        if pass_ is not UNSET:
            field_dict["pass"] = pass_
        if applicable_times is not UNSET:
            field_dict["applicableTimes"] = applicable_times
        if transponders is not UNSET:
            field_dict["transponders"] = transponders

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.fare_pass import FarePass
        from ..models.range_price import RangePrice
        from ..models.single_price import SinglePrice
        from ..models.transponder_system import TransponderSystem

        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        def _parse_price(data: object) -> RangePrice | SinglePrice:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_fare_price_type_0 = SinglePrice.from_dict(data)

                return componentsschemas_fare_price_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_fare_price_type_1 = RangePrice.from_dict(data)

            return componentsschemas_fare_price_type_1

        price = _parse_price(d.pop("price"))

        def _parse_converted_price(data: object) -> RangePrice | SinglePrice | Unset:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_fare_price_type_0 = SinglePrice.from_dict(data)

                return componentsschemas_fare_price_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_fare_price_type_1 = RangePrice.from_dict(data)

            return componentsschemas_fare_price_type_1

        converted_price = _parse_converted_price(d.pop("convertedPrice", UNSET))

        reason = d.pop("reason", UNSET)

        payment_methods = cast(list[str], d.pop("paymentMethods", UNSET))

        _pass_ = d.pop("pass", UNSET)
        pass_: FarePass | Unset
        if isinstance(_pass_, Unset):
            pass_ = UNSET
        else:
            pass_ = FarePass.from_dict(_pass_)

        applicable_times = d.pop("applicableTimes", UNSET)

        _transponders = d.pop("transponders", UNSET)
        transponders: list[TransponderSystem] | Unset = UNSET
        if _transponders is not UNSET:
            transponders = []
            for transponders_item_data in _transponders:
                transponders_item = TransponderSystem.from_dict(transponders_item_data)

                transponders.append(transponders_item)

        fare = cls(
            id=id,
            name=name,
            price=price,
            converted_price=converted_price,
            reason=reason,
            payment_methods=payment_methods,
            pass_=pass_,
            applicable_times=applicable_times,
            transponders=transponders,
        )

        fare.additional_properties = d
        return fare

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AutocompleteResultItemHouseNumberType(str, Enum):
    INTERPOLATED = "interpolated"
    MPA = "MPA"
    PA = "PA"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class PostAutosuggestShowItem(str, Enum):
    DETAILS = "details"
    FUEL = "fuel"
    PHONEMES = "phonemes"
    STREETINFO = "streetInfo"
    TRUCK = "truck"
    TZ = "tz"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error_response import ErrorResponse
from ...models.get_revgeocode_show_item import GetRevgeocodeShowItem
from ...models.get_revgeocode_show_map_references_item import (
    GetRevgeocodeShowMapReferencesItem,
)
from ...models.get_revgeocode_show_nav_attributes_item import (
    GetRevgeocodeShowNavAttributesItem,
)
from ...models.get_revgeocode_show_related_item import GetRevgeocodeShowRelatedItem
from ...models.get_revgeocode_types_item import GetRevgeocodeTypesItem
from ...models.get_revgeocode_with_item import GetRevgeocodeWithItem
from ...models.open_search_reverse_geocode_response import (
    OpenSearchReverseGeocodeResponse,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    at: str | Unset = UNSET,
    bearing: int | Unset = UNSET,
    in_: str | Unset = UNSET,
    types: list[GetRevgeocodeTypesItem] | Unset = UNSET,
    with_: list[GetRevgeocodeWithItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 1,
    political_view: str | Unset = UNSET,
    show_map_references: list[GetRevgeocodeShowMapReferencesItem] | Unset = UNSET,
    show: list[GetRevgeocodeShowItem] | Unset = UNSET,
    show_nav_attributes: list[GetRevgeocodeShowNavAttributesItem] | Unset = UNSET,
    show_related: list[GetRevgeocodeShowRelatedItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(x_request_id, Unset):
        headers["X-Request-ID"] = x_request_id

    params: dict[str, Any] = {}

    params["at"] = at

    params["bearing"] = bearing

    params["in"] = in_

    json_types: list[str] | Unset = UNSET
    if not isinstance(types, Unset):
        json_types = []
        for types_item_data in types:
            types_item = types_item_data.value
            json_types.append(types_item)

    params["types"] = json_types

    json_with_: list[str] | Unset = UNSET
    if not isinstance(with_, Unset):
        json_with_ = []
        for with_item_data in with_:
            with_item = with_item_data.value
            json_with_.append(with_item)

    params["with"] = json_with_

    json_lang: list[str] | Unset = UNSET
    if not isinstance(lang, Unset):
        json_lang = lang

    params["lang"] = json_lang

    params["limit"] = limit

    params["politicalView"] = political_view

    json_show_map_references: list[str] | Unset = UNSET
    if not isinstance(show_map_references, Unset):
        json_show_map_references = []
        for show_map_references_item_data in show_map_references:
            show_map_references_item = show_map_references_item_data.value
            json_show_map_references.append(show_map_references_item)

    params["showMapReferences"] = json_show_map_references

    json_show: list[str] | Unset = UNSET
    if not isinstance(show, Unset):
        json_show = []
        for show_item_data in show:
            show_item = show_item_data.value
            json_show.append(show_item)

    params["show"] = json_show

    json_show_nav_attributes: list[str] | Unset = UNSET
    if not isinstance(show_nav_attributes, Unset):
        json_show_nav_attributes = []
        for show_nav_attributes_item_data in show_nav_attributes:
            show_nav_attributes_item = show_nav_attributes_item_data.value
            json_show_nav_attributes.append(show_nav_attributes_item)

    params["showNavAttributes"] = json_show_nav_attributes

    json_show_related: list[str] | Unset = UNSET
    if not isinstance(show_related, Unset):
        json_show_related = []
        for show_related_item_data in show_related:
            show_related_item = show_related_item_data.value
            json_show_related.append(show_related_item)

    params["showRelated"] = json_show_related

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/revgeocode",
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> ErrorResponse | OpenSearchReverseGeocodeResponse | None:
    if response.status_code == 200:
        response_200 = OpenSearchReverseGeocodeResponse.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = ErrorResponse.from_dict(response.json())

        return response_400

    if response.status_code == 405:
        response_405 = ErrorResponse.from_dict(response.json())

        return response_405

    if response.status_code == 429:
        response_429 = ErrorResponse.from_dict(response.json())

        return response_429

    if response.status_code == 503:
        response_503 = ErrorResponse.from_dict(response.json())

        return response_503

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[ErrorResponse | OpenSearchReverseGeocodeResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    at: str | Unset = UNSET,
    bearing: int | Unset = UNSET,
    in_: str | Unset = UNSET,
    types: list[GetRevgeocodeTypesItem] | Unset = UNSET,
    with_: list[GetRevgeocodeWithItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 1,
    political_view: str | Unset = UNSET,
    show_map_references: list[GetRevgeocodeShowMapReferencesItem] | Unset = UNSET,
    show: list[GetRevgeocodeShowItem] | Unset = UNSET,
    show_nav_attributes: list[GetRevgeocodeShowNavAttributesItem] | Unset = UNSET,
    show_related: list[GetRevgeocodeShowRelatedItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[ErrorResponse | OpenSearchReverseGeocodeResponse]:
    """Reverse Geocode

     This endpoint returns the nearest address to geo coordinates specified in the request.

    Args:
        at (str | Unset):  Example: 52.5308,13.3856.
        bearing (int | Unset):  Example: 42.
        in_ (str | Unset):
        types (list[GetRevgeocodeTypesItem] | Unset):
        with_ (list[GetRevgeocodeWithItem] | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 1.
        political_view (str | Unset):
        show_map_references (list[GetRevgeocodeShowMapReferencesItem] | Unset):
        show (list[GetRevgeocodeShowItem] | Unset):
        show_nav_attributes (list[GetRevgeocodeShowNavAttributesItem] | Unset):
        show_related (list[GetRevgeocodeShowRelatedItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | OpenSearchReverseGeocodeResponse]
    """

    kwargs = _get_kwargs(
        at=at,
        bearing=bearing,
        in_=in_,
        types=types,
        with_=with_,
        lang=lang,
        limit=limit,
        political_view=political_view,
        show_map_references=show_map_references,
        show=show,
        show_nav_attributes=show_nav_attributes,
        show_related=show_related,
        x_request_id=x_request_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    at: str | Unset = UNSET,
    bearing: int | Unset = UNSET,
    in_: str | Unset = UNSET,
    types: list[GetRevgeocodeTypesItem] | Unset = UNSET,
    with_: list[GetRevgeocodeWithItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 1,
    political_view: str | Unset = UNSET,
    show_map_references: list[GetRevgeocodeShowMapReferencesItem] | Unset = UNSET,
    show: list[GetRevgeocodeShowItem] | Unset = UNSET,
    show_nav_attributes: list[GetRevgeocodeShowNavAttributesItem] | Unset = UNSET,
    show_related: list[GetRevgeocodeShowRelatedItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> ErrorResponse | OpenSearchReverseGeocodeResponse | None:
    """Reverse Geocode

     This endpoint returns the nearest address to geo coordinates specified in the request.

    Args:
        at (str | Unset):  Example: 52.5308,13.3856.
        bearing (int | Unset):  Example: 42.
        in_ (str | Unset):
        types (list[GetRevgeocodeTypesItem] | Unset):
        with_ (list[GetRevgeocodeWithItem] | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 1.
        political_view (str | Unset):
        show_map_references (list[GetRevgeocodeShowMapReferencesItem] | Unset):
        show (list[GetRevgeocodeShowItem] | Unset):
        show_nav_attributes (list[GetRevgeocodeShowNavAttributesItem] | Unset):
        show_related (list[GetRevgeocodeShowRelatedItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | OpenSearchReverseGeocodeResponse
    """

    return sync_detailed(
        client=client,
        at=at,
        bearing=bearing,
        in_=in_,
        types=types,
        with_=with_,
        lang=lang,
        limit=limit,
        political_view=political_view,
        show_map_references=show_map_references,
        show=show,
        show_nav_attributes=show_nav_attributes,
        show_related=show_related,
        x_request_id=x_request_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    at: str | Unset = UNSET,
    bearing: int | Unset = UNSET,
    in_: str | Unset = UNSET,
    types: list[GetRevgeocodeTypesItem] | Unset = UNSET,
    with_: list[GetRevgeocodeWithItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 1,
    political_view: str | Unset = UNSET,
    show_map_references: list[GetRevgeocodeShowMapReferencesItem] | Unset = UNSET,
    show: list[GetRevgeocodeShowItem] | Unset = UNSET,
    show_nav_attributes: list[GetRevgeocodeShowNavAttributesItem] | Unset = UNSET,
    show_related: list[GetRevgeocodeShowRelatedItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[ErrorResponse | OpenSearchReverseGeocodeResponse]:
    """Reverse Geocode

     This endpoint returns the nearest address to geo coordinates specified in the request.

    Args:
        at (str | Unset):  Example: 52.5308,13.3856.
        bearing (int | Unset):  Example: 42.
        in_ (str | Unset):
        types (list[GetRevgeocodeTypesItem] | Unset):
        with_ (list[GetRevgeocodeWithItem] | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 1.
        political_view (str | Unset):
        show_map_references (list[GetRevgeocodeShowMapReferencesItem] | Unset):
        show (list[GetRevgeocodeShowItem] | Unset):
        show_nav_attributes (list[GetRevgeocodeShowNavAttributesItem] | Unset):
        show_related (list[GetRevgeocodeShowRelatedItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | OpenSearchReverseGeocodeResponse]
    """

    kwargs = _get_kwargs(
        at=at,
        bearing=bearing,
        in_=in_,
        types=types,
        with_=with_,
        lang=lang,
        limit=limit,
        political_view=political_view,
        show_map_references=show_map_references,
        show=show,
        show_nav_attributes=show_nav_attributes,
        show_related=show_related,
        x_request_id=x_request_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    at: str | Unset = UNSET,
    bearing: int | Unset = UNSET,
    in_: str | Unset = UNSET,
    types: list[GetRevgeocodeTypesItem] | Unset = UNSET,
    with_: list[GetRevgeocodeWithItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 1,
    political_view: str | Unset = UNSET,
    show_map_references: list[GetRevgeocodeShowMapReferencesItem] | Unset = UNSET,
    show: list[GetRevgeocodeShowItem] | Unset = UNSET,
    show_nav_attributes: list[GetRevgeocodeShowNavAttributesItem] | Unset = UNSET,
    show_related: list[GetRevgeocodeShowRelatedItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> ErrorResponse | OpenSearchReverseGeocodeResponse | None:
    """Reverse Geocode

     This endpoint returns the nearest address to geo coordinates specified in the request.

    Args:
        at (str | Unset):  Example: 52.5308,13.3856.
        bearing (int | Unset):  Example: 42.
        in_ (str | Unset):
        types (list[GetRevgeocodeTypesItem] | Unset):
        with_ (list[GetRevgeocodeWithItem] | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 1.
        political_view (str | Unset):
        show_map_references (list[GetRevgeocodeShowMapReferencesItem] | Unset):
        show (list[GetRevgeocodeShowItem] | Unset):
        show_nav_attributes (list[GetRevgeocodeShowNavAttributesItem] | Unset):
        show_related (list[GetRevgeocodeShowRelatedItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | OpenSearchReverseGeocodeResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            at=at,
            bearing=bearing,
            in_=in_,
            types=types,
            with_=with_,
            lang=lang,
            limit=limit,
            political_view=political_view,
            show_map_references=show_map_references,
            show=show,
            show_nav_attributes=show_nav_attributes,
            show_related=show_related,
            x_request_id=x_request_id,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.truck_amenity_showers_type import TruckAmenityShowersType
from ..types import UNSET, Unset

T = TypeVar("T", bound="TruckAmenityShowers")


@_attrs_define
class TruckAmenityShowers:
    """
    Attributes:
        type_ (TruckAmenityShowersType): **BETA, RESTRICTED**

            The kind of amenity

            Description of supported values:

            - **BETA, RESTRICTED** `showers`: showers onsite
        available (bool): **BETA, RESTRICTED**

            When set to:
            - `true` indicates that this amenity is available at this place
            - `false` indicates that this amenity is not available at this place
        number_of_showers (int | Unset): **BETA, RESTRICTED**

            Indicates the number of showers existing at the location.
    """

    type_: TruckAmenityShowersType
    available: bool
    number_of_showers: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        available = self.available

        number_of_showers = self.number_of_showers

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "available": available,
        })
        if number_of_showers is not UNSET:
            field_dict["numberOfShowers"] = number_of_showers

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = TruckAmenityShowersType(d.pop("type"))

        available = d.pop("available")

        number_of_showers = d.pop("numberOfShowers", UNSET)

        truck_amenity_showers = cls(
            type_=type_,
            available=available,
            number_of_showers=number_of_showers,
        )

        truck_amenity_showers.additional_properties = d
        return truck_amenity_showers

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SpeedLimitDirection(str, Enum):
    E = "E"
    N = "N"
    NE = "NE"
    NW = "NW"
    S = "S"
    SE = "SE"
    SW = "SW"
    W = "W"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.place_side_of_street import PlaceSideOfStreet
from ..models.place_type import PlaceType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.location import Location


T = TypeVar("T", bound="Place")


@_attrs_define
class Place:
    """A place represents a generic location relevant for the route.

    Example:
        {'type': 'place', 'location': {'lat': 50.339167, 'lng': 18.93}}

    Attributes:
        type_ (PlaceType): Place type. Each place type can have extra attributes.

            **NOTE:** The list of possible place types could be extended in the future.
            The client application is expected to handle such a case gracefully.
        location (Location): Location on the Earth Example: {'lat': 52.531677, 'lng': 13.381777}.
        name (str | Unset): Location name
        waypoint (int | Unset): If present, this place corresponds to the `via` in the request with the same index.

            Example:

            If the request contains `via=<a>&via=<b>`
            * the place corresponding to `<a>` in the response will have `waypoint: 0`
            * the place corresponding to `<b>` in the response will have `waypoint: 1`.

            Notes:
            * `waypoint` is not present for `origin` and `destination` places. `origin` and `destination` waypoints can be
            trivially identified in the response as the `departure` of the first section and the `arrival` of the final
            section, respectively.
            * `waypoint` is not present for any stops that are added automatically by the router.
        original_location (Location | Unset): Location on the Earth Example: {'lat': 52.531677, 'lng': 13.381777}.
        display_location (Location | Unset): Location on the Earth Example: {'lat': 52.531677, 'lng': 13.381777}.
        side_of_street (PlaceSideOfStreet | Unset): Location of the waypoint or destination, relative to the driving
            direction on the street/road.

            **NOTE:** Based on the original waypoint position and `sideOfStreetHint` waypoint property.
             - If `sideOfStreetHint` property is specified, it takes priority in determining the side of street in the
            response.
             - Nothing is returned in the response in case the side of street is ambiguous (too close to the street),
               or if the position is too far from the street.

            * 'left`: The left side of the street in relation to the driving direction of the route.
            * `right`: The right side of the street in relation to the driving direction of the route.
    """

    type_: PlaceType
    location: Location
    name: str | Unset = UNSET
    waypoint: int | Unset = UNSET
    original_location: Location | Unset = UNSET
    display_location: Location | Unset = UNSET
    side_of_street: PlaceSideOfStreet | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        location = self.location.to_dict()

        name = self.name

        waypoint = self.waypoint

        original_location: dict[str, Any] | Unset = UNSET
        if not isinstance(self.original_location, Unset):
            original_location = self.original_location.to_dict()

        display_location: dict[str, Any] | Unset = UNSET
        if not isinstance(self.display_location, Unset):
            display_location = self.display_location.to_dict()

        side_of_street: str | Unset = UNSET
        if not isinstance(self.side_of_street, Unset):
            side_of_street = self.side_of_street.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "location": location,
        })
        if name is not UNSET:
            field_dict["name"] = name
        if waypoint is not UNSET:
            field_dict["waypoint"] = waypoint
        if original_location is not UNSET:
            field_dict["originalLocation"] = original_location
        if display_location is not UNSET:
            field_dict["displayLocation"] = display_location
        if side_of_street is not UNSET:
            field_dict["sideOfStreet"] = side_of_street

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.location import Location

        d = dict(src_dict)
        type_ = PlaceType(d.pop("type"))

        location = Location.from_dict(d.pop("location"))

        name = d.pop("name", UNSET)

        waypoint = d.pop("waypoint", UNSET)

        _original_location = d.pop("originalLocation", UNSET)
        original_location: Location | Unset
        if isinstance(_original_location, Unset):
            original_location = UNSET
        else:
            original_location = Location.from_dict(_original_location)

        _display_location = d.pop("displayLocation", UNSET)
        display_location: Location | Unset
        if isinstance(_display_location, Unset):
            display_location = UNSET
        else:
            display_location = Location.from_dict(_display_location)

        _side_of_street = d.pop("sideOfStreet", UNSET)
        side_of_street: PlaceSideOfStreet | Unset
        if isinstance(_side_of_street, Unset):
            side_of_street = UNSET
        else:
            side_of_street = PlaceSideOfStreet(_side_of_street)

        place = cls(
            type_=type_,
            location=location,
            name=name,
            waypoint=waypoint,
            original_location=original_location,
            display_location=display_location,
            side_of_street=side_of_street,
        )

        place.additional_properties = d
        return place

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.car_fuel_type import CarFuelType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.fuel_additive import FuelAdditive
    from ..models.fuel_price import FuelPrice


T = TypeVar("T", bound="CarFuel")


@_attrs_define
class CarFuel:
    """
    Attributes:
        type_ (CarFuelType): **BETA, RESTRICTED**

            The type of car fuel

            Description of supported values:

            - **BETA, RESTRICTED** `biodiesel`: bio-diesel
            - **BETA, RESTRICTED** `cng`: compressed natural gas (CNG)
            - **BETA, RESTRICTED** `diesel`: diesel
            - **BETA, RESTRICTED** `diesel_with_additives`: diesel with additives
            - **BETA, RESTRICTED** `e10`: E10 (10% ethanol)
            - **BETA, RESTRICTED** `e20`: E20 (20% ethanol)
            - **BETA, RESTRICTED** `e85`: E85 (minimum 70% ethanol blended gasoline)
            - **BETA, RESTRICTED** `ethanol`: ethanol fuel (when specific type is not known, such as E10, E85)
            - **BETA, RESTRICTED** `ethanol_with_additives`: ethanol with additives
            - **BETA, RESTRICTED** `gasoline`: gasoline
            - **BETA, RESTRICTED** `hvo`: hydrotreated vegetable oil fuel
            - **BETA, RESTRICTED** `hydrogen`: hydrogen
            - **BETA, RESTRICTED** `lng`: liquefied natural gas (LNG)
            - **BETA, RESTRICTED** `lpg`: liquefied petroleum gas (LPG)
            - **BETA, RESTRICTED** `midgrade`: midgrade octane rating
            - **BETA, RESTRICTED** `octane_100`: fuel that consists of 100% octane
            - **BETA, RESTRICTED** `octane_87`: fuel that consists of an octane / gasoline blend with 87% octane / 13%
            gasoline
            - **BETA, RESTRICTED** `octane_89`: fuel that consists of an octane / gasoline blend with 89% octane / 11%
            gasoline
            - **BETA, RESTRICTED** `octane_90`: fuel that consists of an octane / gasoline blend with 90% octane / 10%
            gasoline
            - **BETA, RESTRICTED** `octane_91`: fuel that consists of an octane / gasoline blend with 91% octane / 9%
            gasoline
            - **BETA, RESTRICTED** `octane_92`: fuel that consists of an octane / gasoline blend with 92% octane / 8%
            gasoline
            - **BETA, RESTRICTED** `octane_93`: fuel that consists of octane / gasoline blend with 93% octane / 7% gasoline
            - **BETA, RESTRICTED** `octane_95`: fuel that consists of an octane / gasoline blend with 95% octane / 5%
            gasoline
            - **BETA, RESTRICTED** `octane_98`:  fuel that consists of an octane / gasoline blend with 98% octane / 2%
            gasoline
            - **BETA, RESTRICTED** `premium`: premium octane rating
            - **BETA, RESTRICTED** `regular`: regular octance rating
        available (bool): **BETA, RESTRICTED**

            When set to:
            - `true` indicates that this type of car fuel is available at this place
            - `false` indicates that this type of car fuel is not available at this place
        additives (list[FuelAdditive] | Unset): **BETA, RESTRICTED**

            A list of additional compound fluids sold at the location, related to the specific fuel type
        pressures (list[int] | Unset): **BETA, RESTRICTED**

            Indicates the pressure values in bar for hydrogen fuel at the location
        price (FuelPrice | Unset):
    """

    type_: CarFuelType
    available: bool
    additives: list[FuelAdditive] | Unset = UNSET
    pressures: list[int] | Unset = UNSET
    price: FuelPrice | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        available = self.available

        additives: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.additives, Unset):
            additives = []
            for additives_item_data in self.additives:
                additives_item = additives_item_data.to_dict()
                additives.append(additives_item)

        pressures: list[int] | Unset = UNSET
        if not isinstance(self.pressures, Unset):
            pressures = self.pressures

        price: dict[str, Any] | Unset = UNSET
        if not isinstance(self.price, Unset):
            price = self.price.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "available": available,
        })
        if additives is not UNSET:
            field_dict["additives"] = additives
        if pressures is not UNSET:
            field_dict["pressures"] = pressures
        if price is not UNSET:
            field_dict["price"] = price

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.fuel_additive import FuelAdditive
        from ..models.fuel_price import FuelPrice

        d = dict(src_dict)
        type_ = CarFuelType(d.pop("type"))

        available = d.pop("available")

        _additives = d.pop("additives", UNSET)
        additives: list[FuelAdditive] | Unset = UNSET
        if _additives is not UNSET:
            additives = []
            for additives_item_data in _additives:
                additives_item = FuelAdditive.from_dict(additives_item_data)

                additives.append(additives_item)

        pressures = cast(list[int], d.pop("pressures", UNSET))

        _price = d.pop("price", UNSET)
        price: FuelPrice | Unset
        if isinstance(_price, Unset):
            price = UNSET
        else:
            price = FuelPrice.from_dict(_price)

        car_fuel = cls(
            type_=type_,
            available=available,
            additives=additives,
            pressures=pressures,
            price=price,
        )

        car_fuel.additional_properties = d
        return car_fuel

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ev_availability_evse import EvAvailabilityEvse


T = TypeVar("T", bound="EvAvailabilityStation")


@_attrs_define
class EvAvailabilityStation:
    """
    Attributes:
        id (str | Unset): HERE ID of the station
        cpo_id (str | Unset): The unique ID of the location in the system of the CPO.
        evses (list[EvAvailabilityEvse] | Unset): List of EVSE (electric vehicle supply equipment) in the EV station
    """

    id: str | Unset = UNSET
    cpo_id: str | Unset = UNSET
    evses: list[EvAvailabilityEvse] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        cpo_id = self.cpo_id

        evses: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.evses, Unset):
            evses = []
            for evses_item_data in self.evses:
                evses_item = evses_item_data.to_dict()
                evses.append(evses_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if cpo_id is not UNSET:
            field_dict["cpoId"] = cpo_id
        if evses is not UNSET:
            field_dict["evses"] = evses

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ev_availability_evse import EvAvailabilityEvse

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        cpo_id = d.pop("cpoId", UNSET)

        _evses = d.pop("evses", UNSET)
        evses: list[EvAvailabilityEvse] | Unset = UNSET
        if _evses is not UNSET:
            evses = []
            for evses_item_data in _evses:
                evses_item = EvAvailabilityEvse.from_dict(evses_item_data)

                evses.append(evses_item)

        ev_availability_station = cls(
            id=id,
            cpo_id=cpo_id,
            evses=evses,
        )

        ev_availability_station.additional_properties = d
        return ev_availability_station

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.access import Access
    from ..models.functional_class import FunctionalClass
    from ..models.physical import Physical
    from ..models.speed_limit import SpeedLimit


T = TypeVar("T", bound="NavigationAttributes")


@_attrs_define
class NavigationAttributes:
    """
    Attributes:
        speed_limits (list[SpeedLimit] | Unset): Speed limit for the navigable road following the direction of traffic
            on the street. If traffic is allowed in both directions
            two value sets will be returned. If the request contained a bearing parameter, then the first entry will match
            the returned
            direction.
        functional_class (list[FunctionalClass] | Unset): The value represents one of the five levels:
        access (list[Access] | Unset): The following boolean values are available for access attributes:
        physical (list[Physical] | Unset): The following boolean values are available for physical attributes:
    """

    speed_limits: list[SpeedLimit] | Unset = UNSET
    functional_class: list[FunctionalClass] | Unset = UNSET
    access: list[Access] | Unset = UNSET
    physical: list[Physical] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        speed_limits: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.speed_limits, Unset):
            speed_limits = []
            for speed_limits_item_data in self.speed_limits:
                speed_limits_item = speed_limits_item_data.to_dict()
                speed_limits.append(speed_limits_item)

        functional_class: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.functional_class, Unset):
            functional_class = []
            for functional_class_item_data in self.functional_class:
                functional_class_item = functional_class_item_data.to_dict()
                functional_class.append(functional_class_item)

        access: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.access, Unset):
            access = []
            for access_item_data in self.access:
                access_item = access_item_data.to_dict()
                access.append(access_item)

        physical: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.physical, Unset):
            physical = []
            for physical_item_data in self.physical:
                physical_item = physical_item_data.to_dict()
                physical.append(physical_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if speed_limits is not UNSET:
            field_dict["speedLimits"] = speed_limits
        if functional_class is not UNSET:
            field_dict["functionalClass"] = functional_class
        if access is not UNSET:
            field_dict["access"] = access
        if physical is not UNSET:
            field_dict["physical"] = physical

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.access import Access
        from ..models.functional_class import FunctionalClass
        from ..models.physical import Physical
        from ..models.speed_limit import SpeedLimit

        d = dict(src_dict)
        _speed_limits = d.pop("speedLimits", UNSET)
        speed_limits: list[SpeedLimit] | Unset = UNSET
        if _speed_limits is not UNSET:
            speed_limits = []
            for speed_limits_item_data in _speed_limits:
                speed_limits_item = SpeedLimit.from_dict(speed_limits_item_data)

                speed_limits.append(speed_limits_item)

        _functional_class = d.pop("functionalClass", UNSET)
        functional_class: list[FunctionalClass] | Unset = UNSET
        if _functional_class is not UNSET:
            functional_class = []
            for functional_class_item_data in _functional_class:
                functional_class_item = FunctionalClass.from_dict(
                    functional_class_item_data
                )

                functional_class.append(functional_class_item)

        _access = d.pop("access", UNSET)
        access: list[Access] | Unset = UNSET
        if _access is not UNSET:
            access = []
            for access_item_data in _access:
                access_item = Access.from_dict(access_item_data)

                access.append(access_item)

        _physical = d.pop("physical", UNSET)
        physical: list[Physical] | Unset = UNSET
        if _physical is not UNSET:
            physical = []
            for physical_item_data in _physical:
                physical_item = Physical.from_dict(physical_item_data)

                physical.append(physical_item)

        navigation_attributes = cls(
            speed_limits=speed_limits,
            functional_class=functional_class,
            access=access,
            physical=physical,
        )

        navigation_attributes.additional_properties = d
        return navigation_attributes

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.localized_string import LocalizedString


T = TypeVar("T", bound="SignpostLabelText")


@_attrs_define
class SignpostLabelText:
    """Text on a signpost label.

    Attributes:
        name (LocalizedString | Unset): String with optional language code. Example: {'value': 'Invalidenstraße',
            'language': 'de'}.
    """

    name: LocalizedString | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name: dict[str, Any] | Unset = UNSET
        if not isinstance(self.name, Unset):
            name = self.name.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.localized_string import LocalizedString

        d = dict(src_dict)
        _name = d.pop("name", UNSET)
        name: LocalizedString | Unset
        if isinstance(_name, Unset):
            name = UNSET
        else:
            name = LocalizedString.from_dict(_name)

        signpost_label_text = cls(
            name=name,
        )

        signpost_label_text.additional_properties = d
        return signpost_label_text

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains internal utils for dealing with the client"""

import argparse
import inspect
import json
import logging
import re
import sys
from collections import Counter
from collections.abc import Callable, Iterable, MutableMapping
from enum import Enum
from functools import wraps
from http import HTTPStatus
from typing import Any, Protocol, TypeVar

import flexpolyline.decoding
import numpy as np
import pandas as pd
import shapely
from shapely import Point

from ._errors import CasaGeoError

T = TypeVar("T")

logger = logging.getLogger("casageo.tools")

ietf_bcp47_language_tag_pattern = re.compile(r"[A-Za-z0-9]+(-[A-Za-z0-9]+)*")
iso3166_alpha3_country_code_pattern = re.compile(r"[A-Z]{3}")


class Response(Protocol[T]):
    """A response from an endpoint"""

    status_code: HTTPStatus
    content: bytes
    headers: MutableMapping[str, str]
    parsed: T | None


def validate_ietf_bcp47_language_tag(string: str) -> None:
    if not ietf_bcp47_language_tag_pattern.fullmatch(string):
        raise ValueError(f"Invalid IETF BCP 47 language tag: {string!r}")


def validate_iso3166_alpha3_country_code(string: str) -> None:
    if not iso3166_alpha3_country_code_pattern.fullmatch(string):
        raise ValueError(f"Invalid ISO 3166-1 alpha-3 country code: {string!r}")


def load_json(response: Response[Any]) -> dict:
    if response.status_code != 200:
        raise CasaGeoError(
            f"Bad status code {response.status_code}: {response.content!r}"
        )
    data = json.loads(response.content)
    if "error" in data:
        raise CasaGeoError(data["error"])
    return data


def flexpolyline_points(encoded: str) -> np.ndarray:
    """
    Decode a flexpolyline into an array of points.

    While flexpolyline encodes points as lat-lng, this function returns an array
    of lng-lat points to match GeoPandas.

    If the input contains a third dimension, it will be reflected in the output.
    The type of the third dimension is ignored. The third dimension can be
    removed using ``shapely.force_2d()``.

    Args:
        encoded: The encoded flexpolyline.

    Returns:
        np.ndarray: An array of points with either (lng, lat) or (lng, lat, z)
        coordinates, depending on whether the input contained a third dimension.
    """
    return shapely.points([
        (lng, lat, *z) for lat, lng, *z in flexpolyline.decoding.iter_decode(encoded)
    ])


def point_to_latlng(p: Point) -> str:
    """
    Return a lat-lng string representation of a lng-lat Point object.

    The API expects coordinates in the geographic format.
    """
    return f"{p.y},{p.x}"


def lnglat_to_point(data: str) -> Point:
    # FIXME: This function expects a lng-lat string, while the inverse function
    #        produces a lat-lng string. This should at least be documented!
    try:
        lng, lat = (float(val) for val in data.split(","))
    except ValueError as err:
        raise CasaGeoError(
            f"Bad coordinate string {data!r} a good one looks like this 0.0,0.0"
        ) from err
    return Point(lng, lat)


def implicit_enum_parameters(func):
    """
    Decorator that allows parameters of enum type to accept values of
    their underlying type directly.
    """

    @wraps(func)
    def wrapper(*args, **kwargs):
        signature = inspect.signature(func)
        bound_args = signature.bind(*args, **kwargs)
        for k, v in bound_args.arguments.items():
            param = signature.parameters[k]
            if (
                param != inspect.Parameter.empty
                and isinstance(param.annotation, type)
                and issubclass(param.annotation, Enum)
            ):
                bound_args.arguments[k] = param.annotation(v)

        return func(*bound_args.args, **bound_args.kwargs)

    return wrapper


def duplicates(inp: Iterable[T]) -> list[T]:
    ctr = Counter(inp)
    return [v for v, c in ctr.items() if c > 1]


def cli_throwing_main(
    parser: argparse.ArgumentParser,
    main: Callable[[argparse.Namespace], dict | pd.DataFrame],
):
    args = parser.parse_args()
    if not args.key:
        raise CasaGeoError(
            "Please specify a key in the environment variable CASAGEO_KEY or the --key argument"
        )

    f = sys.stdout
    if args.output:
        f = open(args.output, "w", encoding="utf-8")  # noqa: SIM115

    result = main(args)

    if isinstance(result, dict):
        json.dump(result, f, indent=args.indent, ensure_ascii=False)
    elif isinstance(result, pd.DataFrame):
        result.to_csv(f, sep=";")


def cli_main(
    parser: argparse.ArgumentParser,
    main: Callable[[argparse.Namespace], dict | pd.DataFrame],
):
    try:
        cli_throwing_main(parser, main)
    except Exception as e:
        # this is not pretty, but it will look pretty
        sys.exit(f"{e.__class__.__name__}: {e}")
